import argparse
import os
from pathlib import Path
import shutil
import urllib
import subprocess
import re

asset_dirs = ["artifacts/main", "artifacts/build_python_version"]


def main():

    workdir = Path("build-matlab-toolbox").absolute()

    workdir.mkdir()

    with open("artifacts/main/out-matlab/rrversion.txt", "r") as f:
        tag_name = f.read().strip()

    print(f"tag_name: {tag_name}")

    Path("distfiles").mkdir(exist_ok=True)

    for asset_dir in asset_dirs:
        for fname in list(Path(asset_dir).glob('**/RobotRaconteur-*-MATLAB*')):
            if fname.is_dir():
                continue
            print(fname)
            dest = Path(fname)
            shutil.copy(str(fname), "distfiles/" + dest.name)

    build_dir = Path("build-matlab-toolbox/build")
    matlab_dir = build_dir.joinpath("matlab")
    matlab_dir.mkdir(exist_ok=True, parents=True)

    subprocess.check_call(
        "tar xf ../../../distfiles/RobotRaconteur-*-MATLAB-glnxa64.tar.gz --strip-components 1", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "tar xf ../../../distfiles/RobotRaconteur-*-MATLAB-maci64.tar.gz --strip-components 1 --wildcards RobotRaconteur-*-MATLAB-maci64/RobotRaconteurMex.*", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "tar xf ../../../distfiles/RobotRaconteur-*-MATLAB-maca64.tar.gz --strip-components 1 --wildcards RobotRaconteur-*-MATLAB-maca64/RobotRaconteurMex.*", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "unzip -j ../../../distfiles/RobotRaconteur-*-MATLAB-win64.zip RobotRaconteur-*-MATLAB-win64/RobotRaconteurMex.mexw64", shell=True, cwd=matlab_dir)

    semver_regex = r"^((?:0|[1-9]\d*)\.(?:0|[1-9]\d*)\.(?:0|[1-9]\d*))(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"

    print(tag_name)
    semver_match = re.match(semver_regex, tag_name)

    toolbox_ver = semver_match.group(1)

    with open("robotraconteur/packaging/matlab/RobotRaconteurMatlab.prj.in") as f1:
        s1 = f1.read()
    with open(build_dir.joinpath("RobotRaconteurMatlab.prj"), "w") as f2:
        f2.write(s1.replace("$(rr_version)", toolbox_ver))

    shutil.copy("robotraconteur/packaging/matlab/logo-icon.png", build_dir)


if __name__ == "__main__":
    main()



from __future__ import absolute_import

import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s
import pytest


class ServiceTestClient3:
    def __init__(self):
        pass

    def RunFullTest(self, url):
        self.ConnectService(url)

        self.TestProperties()

        self.TestFunctions()

        self.DisconnectService()

    def ConnectService(self, url):
        self._r = RRN.ConnectService(url)

    def DisconnectService(self):
        RRN.DisconnectService(self._r)

    def TestProperties(self):
        _ = self._r.d1
        self._r.d1 = 3.0819

        try:
            _ = self._r.err
        except RR.InvalidArgumentException as e:
            assert e.message == "Test message 1"
        else:
            assert False

        with pytest.raises(RR.InvalidOperationException):
            self._r.err = 100

    def TestFunctions(self):

        self._r.f1()
        self._r.f2(247)

        res = self._r.f3(1, 2)
        assert res == 3

        with pytest.raises(RR.InvalidOperationException):
            self._r.err_func()

        asynctestexp = RRN.GetExceptionType(
            "com.robotraconteur.testing.TestService5.asynctestexp", self._r)
        try:
            self._r.err_func2()
        except asynctestexp:
            pass
        else:
            assert False



# varvalue_value_types.py - Example of using varvalue wildcard type

from RobotRaconteur.Client import *
import numpy as np

c = RRN.ConnectService('rr+tcp://localhost:53223?service=values_example')

# Any valid Robot Raconteur data type can be stored in a varvalue
# In Python, the RR.VarValue() Python structure is used to store
# the value and type of the data. The type is stored as a string.
# This is necessary because Python does not carry enough type
# information

# property varvalue p_varvalue_double_array
# Example of storing a double array in a varvalue
a = np.array([1.1, 2.2, 3.3], dtype=np.float64)
c.p_varvalue_double_array = RR.VarValue(a, "double[]")
b = c.p_varvalue_double_array
np.testing.assert_array_equal(b.data, [1.4, 2.5, 3.6])
assert b.datatype == "double[]"

# property varvalue q_varvalue_string
# Example of storing a string in a varvalue
c.q_varvalue_string = RR.VarValue("varvalue string from client", "string")
d = c.q_varvalue_string
assert d.data == "varvalue string from service"
assert d.datatype == "string"

# property varvalue r_varvalue_struct
# Example of storing a structure in a varvalue
my_structure_type = RRN.GetStructureType("experimental.value_types.MyStructure", obj=c)
s = my_structure_type()
s.a = 5
s.b = np.array([10, 20], dtype=np.uint32)
s.c = "String from structure client"
s.d = [
    "string a",
    "string b"
]

c.r_varvalue_struct = RR.VarValue(s, "experimental.value_types.MyStructure")

t = c.r_varvalue_struct
t_data = t.data

assert t_data.a == 52
np.testing.assert_array_equal(t_data.b, np.array([110, 120], dtype=np.uint32))
assert t_data.c == "String from structure service"
assert len(t_data.d) == 3
assert t_data.d[0] == "string c"
assert t_data.d[1] == "string d"
assert t_data.d[2] == "string e"

# property varvalue{string} s_varvalue_map2
# Example of storing a map of strings in a varvalue
# varvalue{string} is used extensively in structures
# to allow for additional fields to be added to the
# structure without changing the structure definition
c.s_varvalue_map2 = {
    "key1": RR.VarValue(np.array([2, 3]), "int32[]"),
    "key2": RR.VarValue("string 2", "string")
}

u = c.s_varvalue_map2
assert len(u) == 2
np.testing.assert_array_equal(u["key3"].data, [4, 5])
assert u["key3"].datatype == "int32[]"
assert u["key4"].data == "string 4"
assert u["key4"].datatype == "string"



from RobotRaconteur.Client import *
import time
import numpy as np

# Connect to the Reynard service using a URL
c = RRN.ConnectService('rr+tcp://localhost:29200?service=reynard')


def new_message(msg):
    print(f"New message: {msg}")


# Connect a callback function to listen for new messages
c.new_message += new_message

# Read the current state using a wire "peek". Can also "connect" to receive streaming updates.
state, _ = c.state.PeekInValue()
print(state)

# Teleport the robot
c.teleport(0.1, -0.2)

# Drive the robot with no timeout
c.drive_robot(0.5, -0.2, -1, False)

# Wait for one second
time.sleep(1)

# Stop the robot
c.drive_robot(0, 0, -1, False)

# Set the arm position
c.setf_arm_position(np.deg2rad(100), np.deg2rad(-30), np.deg2rad(-70))

# Drive the arm using timeout and wait
c.drive_arm(np.deg2rad(10), np.deg2rad(-30), np.deg2rad(-15), 1.5, True)

# Set the color to red
c.color = (1, 0, 0)

# Read the color
print(f"Color: {c.color}")

time.sleep(1)

# Reset the color
c.color = (0.929, 0.49, 0.192)

# Say hello
c.say("Hello, World From Robot Raconteur!")



# device_connector_yaml.py - Example of using the DeviceConnector class to connect to a device

# To run this example:
#    python device_connector_yml.py --devices devices.yaml

from RobotRaconteur.Client import *
from RobotRaconteurCompanion.Util.DeviceConnector import DeviceConnector, DeviceConnectorDetails
from RobotRaconteurCompanion.Util.ImageUtil import ImageUtil
import cv2
import argparse

# Parse command line arguments to get the devices yaml file object
parser = argparse.ArgumentParser(description="Device connector example")
parser.add_argument("--devices", type=argparse.FileType("r"), required=True, help="The devices yaml file")
args, _ = parser.parse_known_args()

# Create a DeviceConnector object and load the devices yaml file
con = DeviceConnector(devices_yaml_f=args.devices)

# Get the ServiceSubscription to the camera
sub = con.GetDevice("camera")

# Get the default client and capture and image
c = sub.GetDefaultClientWait(10)
raw_img = c.capture_frame()
image_util = ImageUtil(client_obj=c)

# Display the image
img = image_util.image_to_array(raw_img)
cv2.namedWindow("Image")
cv2.imshow("Image", img)
cv2.waitKey()
cv2.destroyAllWindows()



from RobotRaconteur.Client import *
import sys
import time

try:
    raw_input  # type: ignore
except NameError:
    raw_input = input


def main():

    if (len(sys.argv) < 2):
        print("Usage for subscriberurltest:  subscriberurltest.py url")
        return

    url = sys.argv[1]

    print(url)

    s = RRN.SubscribeService(url)

    def connected(s, client_id, client):
        print("Client connected: " + str(client_id.NodeID) +
              "," + client_id.ServiceName)

    def disconnected(s, client_id, client):
        print("Client disconnected: " + str(client_id.NodeID) +
              "," + client_id.ServiceName)

    def connect_failed(s, client_id, url, err):
        print("Client connect failed: " + str(client_id.NodeID) +
              " url: " + str(url) + " error: " + str(err))
    s.ClientConnected += connected
    s.ClientDisconnected += disconnected
    s.ClientConnectFailed += connect_failed

    def async_get_handler(obj, err):
        if err is not None:
            print("AsyncGetDefaultClient error: " + str(err))
        else:
            print("AsyncGetDefaultClient success: " + str(obj))

    s.AsyncGetDefaultClient(async_get_handler, 1)
    client1 = s.GetDefaultClientWait(6)
    print(s.TryGetDefaultClientWait(6))
    print(s.GetConnectedClients())
    try:
        print(s.GetDefaultClient().d1)
    except:
        print("Client not connected")
    print(s.TryGetDefaultClient())
    raw_input("Press enter")


if __name__ == "__main__":
    main()



import sys
import numpy as np
import struct


try:
    xrange
except NameError:
    xrange = range


def _clip_uint32(x):
    return x & 0xFFFFFFFF


def _robotraconteur_test_lfsr_next(lfsr):
    bit = ((lfsr >> 0) ^ (lfsr >> 1) ^ (lfsr >> 21) ^ (lfsr >> 31)) & 1
    return _clip_uint32((lfsr >> 1) | (bit << 31))


def _robotraconteur_test_lfsr_initkey(seed, key):

    # Run the LFSR a few times on the seed to shuffle
    for i in xrange(128):
        seed = _robotraconteur_test_lfsr_next(seed)

    i = 0
    hash_ = _robotraconteur_test_lfsr_next(seed)
    while (i != len(key)):
        hash_ = _clip_uint32(hash_ + ord(key[i]))
        i += 1
        hash_ = _clip_uint32(hash_ + _clip_uint32(hash_ << 10))
        hash_ = hash_ ^ _clip_uint32(hash_ >> 6)

    hash_ = _clip_uint32(hash_ + _clip_uint32(hash_ << 3))
    hash_ = hash_ ^ (hash_ >> 11)
    hash_ = _clip_uint32(hash_ + _clip_uint32(hash_ << 15))

    # Shuffle again
    for i in xrange(128):
        hash_ = _robotraconteur_test_lfsr_next(hash_)

    return _robotraconteur_test_lfsr_next(hash_)


def _robotraconteur_test_lfsr_next_char(lfsr):
    chars = " !\"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
    lfsr2 = _robotraconteur_test_lfsr_next(lfsr)
    ind_ = lfsr2 % len(chars)
    return lfsr2, chars[ind_]


def _robotraconteur_test_lfsr_next_len(lfsr, max_len):
    lfsr2 = _robotraconteur_test_lfsr_next(lfsr)
    if (max_len < 32):
        len_out = lfsr2 % max_len
    else:
        len_out = 8 + lfsr2 % (max_len - 8)
    return lfsr2, len_out


def _robotraconteur_test_lfsr_next_dist(lfsr, min_, max_):
    assert (max_ > min_)
    lfsr2 = _robotraconteur_test_lfsr_next(lfsr)
    val_out = (lfsr2 % ((max_ - min_) + 1)) + min_
    return lfsr2, val_out


def _reinterpret_cast(fmt, x):
    b = struct.pack("<I", x)
    return struct.unpack_from("<" + fmt, b)[0]


def _next_typed(lfsr, fmt):
    lfsr2 = _robotraconteur_test_lfsr_next(lfsr)
    val_out = _reinterpret_cast(fmt, lfsr2)
    return lfsr2, val_out


def _reinterpret_cast2(fmt, x):
    b = struct.pack("<Q", x)
    return struct.unpack_from("<" + fmt, b)[0]


def _next_typed2(lfsr, fmt):
    lfsr2 = _robotraconteur_test_lfsr_next(lfsr)
    lfsr3 = _robotraconteur_test_lfsr_next(lfsr2)
    val_uint64 = (lfsr2 << 32) | lfsr3
    val_out = _reinterpret_cast2(fmt, val_uint64)
    return lfsr3, val_out


def _next_array(fn, dtype, len_):
    ret = np.zeros((len_), dtype=dtype)
    for i in xrange(len_):
        ret[i] = fn()
    return ret


def _next_array_var_len(fn, dtype, max_len, len_fn):
    len_ = len_fn(max_len)
    return _next_array(fn, dtype, len_)


class LFSRSeqGen:
    def __init__(self, seed, key=None):
        if key is None:
            self.lfsr = seed
        else:
            self.lfsr = _robotraconteur_test_lfsr_initkey(seed, key)

    def _next_typed(self, fmt):
        self.lfsr, val_out = _next_typed(self.lfsr, fmt)
        return val_out

    def _next_typed2(self, fmt):
        self.lfsr, val_out = _next_typed2(self.lfsr, fmt)
        return val_out

    def next_int8(self):
        return self._next_typed("b")

    def next_uint8(self):
        return self._next_typed("B")

    def next_int16(self):
        return self._next_typed("h")

    def next_uint16(self):
        return self._next_typed("H")

    def next_int32(self):
        return self._next_typed("i")

    def next_uint32(self):
        return self._next_typed("I")

    def next_int64(self):
        return self._next_typed2("q")

    def next_uint64(self):
        return self._next_typed2("Q")

    def next_float(self):
        return self._next_typed("f")

    def next_double(self):
        return self._next_typed2("d")

    def next_bool(self):
        return (self._next_typed("I") & 0x1) != 0

    def next_cfloat(self):
        return self._next_typed("f") + (self._next_typed("f") * 1j)

    def next_cdouble(self):
        return self._next_typed2("d") + (self._next_typed2("d") * 1j)

    def next_char(self):
        self.lfsr, val_out = _robotraconteur_test_lfsr_next_char(self.lfsr)
        return val_out

    def next_len(self, max_len):
        self.lfsr, val_out = _robotraconteur_test_lfsr_next_len(
            self.lfsr, max_len)
        return val_out

    def next_dist(self, min_, max_):
        self.lfsr, val_out = _robotraconteur_test_lfsr_next_dist(
            self.lfsr, min_, max_)
        return val_out

    def next_int8_array(self, len_):
        return _next_array(self.next_int8, np.int8, len_)

    def next_uint8_array(self, len_):
        return _next_array(self.next_uint8, np.uint8, len_)

    def next_int16_array(self, len_):
        return _next_array(self.next_int16, np.int16, len_)

    def next_uint16_array(self, len_):
        return _next_array(self.next_uint16, np.uint16, len_)

    def next_int32_array(self, len_):
        return _next_array(self.next_int32, np.int32, len_)

    def next_uint32_array(self, len_):
        return _next_array(self.next_uint32, np.uint32, len_)

    def next_int64_array(self, len_):
        return _next_array(self.next_int64, np.int64, len_)

    def next_uint64_array(self, len_):
        return _next_array(self.next_uint64, np.uint64, len_)

    def next_float_array(self, len_):
        return _next_array(self.next_float, np.float32, len_)

    def next_double_array(self, len_):
        return _next_array(self.next_double, np.float64, len_)

    def next_bool_array(self, len_):
        return _next_array(self.next_bool, np.bool8, len_)

    def next_cfloat_array(self, len_):
        return _next_array(self.next_cfloat, np.complex64, len_)

    def next_cdouble_array(self, len_):
        return _next_array(self.next_cdouble, np.complex128, len_)

    def next_string(self, len_):
        out = ""
        for _ in xrange(len_):
            out += self.next_char()
        return out

    def next_int8_array_var_len(self, max_len):
        return _next_array_var_len(self.next_int8, np.int8, max_len, self.next_len)

    def next_uint8_array_var_len(self, max_len):
        return _next_array_var_len(self.next_uint8, np.uint8, max_len, self.next_len)

    def next_int16_array_var_len(self, max_len):
        return _next_array_var_len(self.next_int16, np.int16, max_len, self.next_len)

    def next_uint16_array_var_len(self, max_len):
        return _next_array_var_len(self.next_uint16, np.int16, max_len, self.next_len)

    def next_int32_array_var_len(self, max_len):
        return _next_array_var_len(self.next_int32, np.int32, max_len, self.next_len)

    def next_uint32_array_var_len(self, max_len):
        return _next_array_var_len(self.next_uint32, np.uint32, max_len, self.next_len)

    def next_int64_array_var_len(self, max_len):
        return _next_array_var_len(self.next_int64, np.int64, max_len, self.next_len)

    def next_uint64_array_var_len(self, max_len):
        return _next_array_var_len(self.next_uint64, np.uint64, max_len, self.next_len)

    def next_float_array_var_len(self, max_len):
        return _next_array_var_len(self.next_float, np.float32, max_len, self.next_len)

    def next_double_array_var_len(self, max_len):
        return _next_array_var_len(self.next_double, np.float64, max_len, self.next_len)

    def next_bool_array_var_len(self, max_len):
        return _next_array_var_len(self.next_bool, np.bool8, max_len, self.next_len)

    def next_cfloat_array_var_len(self, max_len):
        return _next_array_var_len(self.next_cfloat, np.complex64, max_len, self.next_len)

    def next_cdouble_array_var_len(self, max_len):
        return _next_array_var_len(self.next_cdouble, np.complex128, max_len, self.next_len)

    def next_string_var_len(self, max_len):
        len_ = self.next_len(max_len)
        out = ""
        for _ in xrange(len_):
            out += self.next_char()
        return out


if __name__ == "__main__":

    count = 8
    seed = 35913226
    key = "Hello World!"

    if (len(sys.argv) > 1):
        key = sys.argv[1]

    seqgen = LFSRSeqGen(seed, key)

    print(seqgen.next_int8_array(count))
    print(seqgen.next_uint8_array(count))
    print(seqgen.next_int16_array(count))
    print(seqgen.next_uint16_array(count))
    print(seqgen.next_int32_array(count))
    print(seqgen.next_uint32_array(count))
    print(seqgen.next_int64_array(count))
    print(seqgen.next_uint64_array(count))
    print(seqgen.next_float_array(count))
    print(seqgen.next_double_array(count))
    print(seqgen.next_string(count))
    print(seqgen.next_bool_array(count))
    print(seqgen.next_cfloat_array(count))
    print(seqgen.next_cdouble_array(count))
    print()
    print(seqgen.next_int8_array_var_len(count))
    print(seqgen.next_uint8_array_var_len(count))
    print(seqgen.next_int16_array_var_len(count))
    print(seqgen.next_uint16_array_var_len(count))
    print(seqgen.next_int32_array_var_len(count))
    print(seqgen.next_uint32_array_var_len(count))
    print(seqgen.next_int64_array_var_len(count))
    print(seqgen.next_uint64_array_var_len(count))
    print(seqgen.next_float_array_var_len(count))
    print(seqgen.next_double_array_var_len(count))
    print(seqgen.next_string_var_len(count))
    print(seqgen.next_bool_array_var_len(count))
    print(seqgen.next_cfloat_array_var_len(count))
    print(seqgen.next_cdouble_array_var_len(count))



# namedarray_value_types.py - Example of using namedarray value types

from RobotRaconteur.Client import *
import numpy as np

c = RRN.ConnectService('rr+tcp://localhost:53223?service=values_example')

# Retrieve NamedArray numpy dtypes. NamedArrays are represented as NumPy dtypes with fields
# obj is only used for clients. It is omitted for services.
my_vector3_dtype = RRN.GetNamedArrayDType("experimental.value_types.MyVector3", obj=c)
my_quaternion_dtype = RRN.GetNamedArrayDType("experimental.value_types.MyQuaternion", obj=c)
my_pose_dtype = RRN.GetNamedArrayDType("experimental.value_types.MyPose", obj=c)

# NumPy structures are used to represent NamedArrays.
# These structures can also be converted to and from plain arrays of the
# underlying numeric type using RRN.NamedArrayToArray() and RRN.ArrayToNamedArray().
# Scalar NamedArrays are represented as arrays with a single element.

# property MyVector3 g_vector
a = np.zeros((1,), dtype=my_vector3_dtype)
a[0]["x"] = 1.0
a[0]["y"] = 2.0
a[0]["z"] = 3.0
c.g_vector = a

b = c.g_vector
assert (b[0]["x"] == 4.0)
assert (b[0]["y"] == 5.0)
assert (b[0]["z"] == 6.0)

# Use plain arrays and convert to and from NamedArrays
d = np.array([1.0, 2.0, 3.0], dtype=np.float64)
c.g_vector = RRN.ArrayToNamedArray(d, named_array_dt=my_vector3_dtype)
e = RRN.NamedArrayToArray(c.g_vector)
np.testing.assert_array_equal(e, [[4.0, 5.0, 6.0]])

# Now an array of NamedArrays
# property MyVector3[] g_vector_array
f = np.zeros((2,), dtype=my_vector3_dtype)
f[0]["x"] = 1.0
f[0]["y"] = 2.0
f[0]["z"] = 3.0
f[1]["x"] = 4.0
f[1]["y"] = 5.0
f[1]["z"] = 6.0
c.g_vector_array = f

g = c.g_vector_array
assert (g[0]["x"] == 7.0)
assert (g[0]["y"] == 8.0)
assert (g[0]["z"] == 9.0)
assert (g[1]["x"] == 10.0)
assert (g[1]["y"] == 11.0)
assert (g[1]["z"] == 12.0)

# Use plain arrays and convert to and from NamedArrays
h = np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], dtype=np.float64)
c.g_vector_array = RRN.ArrayToNamedArray(h, named_array_dt=my_vector3_dtype)
i = RRN.NamedArrayToArray(c.g_vector_array)
np.testing.assert_array_equal(i, [[7.0, 8.0, 9.0], [10.0, 11.0, 12.0]])

# property MyQuaternion h_quaternion
j = np.zeros((1,), dtype=my_quaternion_dtype)
j[0]["w"] = 1.0
j[0]["x"] = 0.0
j[0]["y"] = 0.0
j[0]["z"] = 0.0
c.h_quaternion = j

k = c.h_quaternion
assert (k[0]["w"] == 0.707)
assert (k[0]["x"] == 0.0)
assert (k[0]["y"] == 0.707)
assert (k[0]["z"] == 0.0)

# MyPose is a composite NamedArray with fields of other NamedArrays
# property MyPose l_pose
l = np.zeros((1,), dtype=my_pose_dtype)
l[0]["orientation"]["w"] = 1.0
l[0]["orientation"]["x"] = 0.0
l[0]["orientation"]["y"] = 0.0
l[0]["orientation"]["z"] = 0.0
l[0]["position"]["x"] = 1.0
l[0]["position"]["y"] = 2.0
l[0]["position"]["z"] = 3.0
c.i_pose = l

m = c.i_pose
assert (m[0]["orientation"]["w"] == 0.707)
assert (m[0]["orientation"]["x"] == 0.0)
assert (m[0]["orientation"]["y"] == 0.707)
assert (m[0]["orientation"]["z"] == 0.0)
assert (m[0]["position"]["x"] == 4.0)
assert (m[0]["position"]["y"] == 5.0)
assert (m[0]["position"]["z"] == 6.0)

# Use plain arrays and convert to and from NamedArrays
n = np.array([1.0, 0.0, 0.0, 0.0, 1.0, 2.0, 3.0,], dtype=np.float64)
c.i_pose = RRN.ArrayToNamedArray(n, named_array_dt=my_pose_dtype)
o = RRN.NamedArrayToArray(c.i_pose)
np.testing.assert_array_equal(o, [[0.707, 0.0, 0.707, 0.0, 4.0, 5.0, 6.0]])



from RobotRaconteur.Client import *
import sys
import time

# cSpell: ignore rrname


def main():
    rrname = sys.argv[1]
    transports = sys.argv[2].split(",")

    time.sleep(6)

    ret = RRN.FindNodeByName(rrname, transports)

    for r in ret:
        print_NodeInfo2(r)

    def printret(ret):
        for r in ret:
            print_NodeInfo2(r)
    RRN.AsyncFindNodeByName(rrname, transports, printret, 1)

    time.sleep(2)


def print_NodeInfo2(s):
    print("NodeID: " + str(s.NodeID))
    print("NodeName: " + s.NodeName)
    print("ConnectionURL: " + str(s.ConnectionURL))


if __name__ == "__main__":
    main()



# Copyright 2011-2020 Wason Technology, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import numpy
import sys

if sys.version_info[0] >= 3:
    import reprlib


class EventHook(object):
    """
    EventHook is used to implement multiple listener events in
    Robot Raconteur Python. Callback functions are registered with the
    EventHook and are all notified when the event is fired.
    """
    __slots__ = "__handlers"

    def __init__(self):
        """
        Initialize new instance
        """
        self.__handlers = []

    def __iadd__(self, handler):
        """
        Adds a callback function to be notified of events
        """
        self.__handlers.append(handler)
        return self

    def __isub__(self, handler):
        """
        Removes a callback function
        """
        self.__handlers.remove(handler)
        return self

    def fire(self, *args, **kwargs):
        """
        Fires the event
        """
        for handler in self.__handlers:
            handler(*args, **kwargs)

    def clearObjectHandlers(self, inObject):
        for theHandler in self.__handlers:
            if theHandler.im_self == inObject:
                self -= theHandler


class RobotRaconteurStructure(object):
    __slots__ = []

    if sys.version_info[0] >= 3:
        @reprlib.recursive_repr()
        def __repr__(self):
            return '{}({})'.format(self.__class__.__name__, ', '.join(['{}={}'.format(k,
                                                                                      repr(getattr(self, k))) for k in self.__slots__]))


class RobotRaconteurVarValue(object):
    __slots__ = ["data", "datatype"]

    def __init__(self, data, datatype):
        self.data = data
        """The data stored in the ``varvalue``"""
        self.datatype = datatype
        """(str,RobotRaconteur.TypeDefinition) The type of the ``varvalue`` data"""

    def __str__(self):
        return str(self.datatype) + ": " + str(self.data)

    def __repr__(self):
        return "RobotRaconteurVarValue: " + str(self.datatype) + ": " + repr(self.data)


VarValue = RobotRaconteurVarValue


class ArrayMemory(object):
    """
    The ``ArrayMemory`` is designed to represent a large array that is read in smaller
    pieces. It is used with the \"memory\" member to allow for random access to an array.
    """
    __slots__ = ["memory"]

    def __init__(self, memory=None):
        self.memory = memory
        """(numpy.ndarray) The underlying array of the memory"""

    def Attach(self, memory):
        """
        Attach the memory to an array

        :param memory: The data to attach
        :type memory: numpy.ndarray
        """
        self.memory = memory

    @property
    def Length(self):
        """
        The length of the memory

        :rtype: int
        """
        return len(self.memory)

    def Read(self, memorypos, buffer, bufferpos, count):
        """
        Reads data from the memory into ``buffer``

        :param memorypos: The start position in the memory array
        :type memorypos: int
        :param buffer: The buffer to read the data into
        :type buffer: numpy.ndarray
        :param bufferpos: The start position in the supplied buffer
        :type bufferpos: int
        :param count: The number of elements to read
        :type count: int
        """
        buffer[bufferpos:(bufferpos + count)
               ] = self.memory[memorypos:(memorypos + count)]

    def Write(self, memorypos, buffer, bufferpos, count):
        """
        Writes data from ``buffer`` into the memory

        :param memorypos: The start position in the memory array
        :type memorypos: int
        :param buffer: The buffer to write the data from
        :type buffer: numpy.ndarray
        :param bufferpos: The start position in the supplied buffer
        :type bufferpos: int
        :param count: The number of elements to write
        :type count: int
        """
        self.memory[memorypos:(memorypos + count)
                    ] = buffer[bufferpos:(bufferpos + count)]


class MultiDimArrayMemory(object):
    """
    The ``MultiDimArrayMemory`` is designed to represent a large
    multi-dimensional array that is read in smaller pieces. It is used with the
    \"memory\" member to allow for random access to an multi-dimensional array.  It works with
    numpy.ndarray.  For the ``memorypos``, ``bufferpos``, and ``count`` parameters in the
    functions, a list is used to represent the dimensions.  The dimensions are column-major
    as is numpy.ndarray.
    """
    __slots__ = ["memory"]

    def __init__(self, memory=None):
        if (memory is None):
            self.memory = None
            """(numpy.ndarray) The underlying array of the memory"""
        else:
            self.Attach(memory)

    def Attach(self, memory):
        """
        Attach the memory to an array

        :param memory: The data to attach
        :type memory: numpy.ndarray
        """
        if (not isinstance(memory, numpy.ndarray)):
            raise Exception(
                "MultiDimArrayMemory memory must be numpy.ndarray type")
        self.memory = memory

    @property
    def Dimensions(self):
        """
        The dimensions of the memory

        :rtype: List[int]
        """
        return self.memory.shape

    @property
    def DimCount(self):
        """
        The number of dimensions

        :rtype: int
        """
        return self.memory.ndim

    def Read(self, memorypos, buffer, bufferpos, count):
        """
        Reads data from the memory into ``buffer``

        :param memorypos: The start position in the array
        :type memorypos: List[int]
        :param buffer: The buffer to read the data into
        :type buffer: numpy.ndarray
        :param bufferpos: The start position int he buffer
        :type bufferpos: List[int]
        :param count: The shape of elements to read
        :type count: List[int]
        """
        memind = [slice(memorypos[i], (memorypos[i] + count[i]))
                  for i in range(len(count))]
        bufind = [slice(bufferpos[i], (bufferpos[i] + count[i]))
                  for i in range(len(count))]
        buffer[tuple(bufind)] = self.memory[tuple(memind)]

    def Write(self, memorypos, buffer, bufferpos, count):
        """
        Writes data into the memory from ``buffer``

        :param memorypos: The start position in the array
        :type memorypos: List[int]
        :param buffer: The buffer to write the data from
        :type buffer: numpy.ndarray
        :param bufferpos: The start position in the buffer
        :type bufferpos: List[int]
        :param count: The shape of elements to write
        :type count: List[int]
        """
        memind = [slice(memorypos[i], (memorypos[i] + count[i]))
                  for i in range(len(count))]
        bufind = [slice(bufferpos[i], (bufferpos[i] + count[i]))
                  for i in range(len(count))]
        self.memory[tuple(memind)] = buffer[tuple(bufind)]


bool_dtype = numpy.uint8

from . import RobotRaconteurPython



import os
from RobotRaconteurTest.service_test_util import TestServerNodeConfig
import pytest


def pytest_generate_tests(metafunc):
    robdef_path = os.path.abspath(os.path.dirname(
        os.path.abspath(__file__)) + "/../robdef")
    os.environ["ROBOTRACONTEUR_ROBDEF_PATH"] = robdef_path
    os.environ["ROBOTRACONTEUR_PYTHON_TRACEBACK_PRINT_EXC"] = "TRUE"
    # os.environ["ROBOTRACONTEUR_PYTHON_ENABLE_DEBUGPY"] = "TRUE"
    os.environ["ROBOTRACONTEUR_LOG_LEVEL"] = "ERROR"


@pytest.fixture(scope="session")
def test_server_node_config():
    return TestServerNodeConfig("unit_test_service_python")



import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s
import numpy as np


class ValueTypesExample_impl:
    # Scalar numbers
    @property
    def a_double(self):
        return 5.78

    @a_double.setter
    def a_double(self, value):
        assert (value == 49.3)

    @property
    def b_int(self):
        return 4557

    @b_int.setter
    def b_int(self, value):
        assert (value == 359)

    @property
    def c_byte(self):
        return 0x1A

    @c_byte.setter
    def c_byte(self, value):
        assert (value == 31)

    @property
    def d_cdouble(self):
        return 23.7 + 1j * 5.3

    @d_cdouble.setter
    def d_cdouble(self, value):
        assert (value == 1.2 + 3.4j)

    @property
    def e_bool(self):
        return True

    @e_bool.setter
    def e_bool(self, value):
        assert (value == False)

    @property
    def meaning_of_life(self):
        return 42

    # Numeric Arrays
    @property
    def a_double_array(self):
        return [0.016, 0.226]

    @a_double_array.setter
    def a_double_array(self, value):
        np.testing.assert_allclose(value, [0.582, 0.288, 0.09, 0.213, 0.98])

    @property
    def a_double_array_fixed(self):
        return [0.13, 0.27, 0.15]

    @a_double_array_fixed.setter
    def a_double_array_fixed(self, value):
        np.testing.assert_allclose(value, [0.21, 0.12, 0.39])

    @property
    def a_double_array_maxlen(self):
        return [0.7, 0.16, 0.16, 0.05, 0.61, 0.9]

    @a_double_array_maxlen.setter
    def a_double_array_maxlen(self, value):
        np.testing.assert_allclose(value, [0.035, 0.4])

    @property
    def a_double_marray_fixed(self):
        return np.array([[0.29, 0.66], [0.41, 0.6], [0.4, 0.2]], dtype=np.float64)

    @a_double_marray_fixed.setter
    def a_double_marray_fixed(self, value):
        np.testing.assert_allclose(value, [[0.3, 0.6], [0.4, 0.6], [0.5, 0.2]])

    @property
    def a_double_marray(self):
        return np.array([[0.72, 0.4], [0.05, 0.07]], dtype=np.float64)

    @a_double_marray.setter
    def a_double_marray(self, value):
        np.testing.assert_allclose(value, [[0.3], [0.01]])

    @property
    def c_byte_array(self):
        return [0x1A, 0x2B]

    @c_byte_array.setter
    def c_byte_array(self, value):
        np.testing.assert_array_equal(value, [0x3C, 0x4D, 0x5E, 0x6F, 0x70])

    # Strings
    @property
    def f_string(self):
        return "An example string read from the service"

    @f_string.setter
    def f_string(self, value):
        assert (value == "An example string written to the service")

    # Structures
    @property
    def k_struct(self):
        s = RRN.NewStructure("experimental.value_types.MyStructure")
        fill_MyStructure(s)
        return s

    @k_struct.setter
    def k_struct(self, value):
        assert_MyStructure(value)

    # NamedArrays
    @property
    def g_vector(self):
        a = np.zeros((1,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyVector3"))
        a[0]["x"] = 4.0
        a[0]["y"] = 5.0
        a[0]["z"] = 6.0
        return a

    @g_vector.setter
    def g_vector(self, value):
        assert (value[0]["x"] == 1.0)
        assert (value[0]["y"] == 2.0)
        assert (value[0]["z"] == 3.0)

    @property
    def g_vector_array(self):
        f = np.zeros((2,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyVector3"))
        f[0]["x"] = 7.0
        f[0]["y"] = 8.0
        f[0]["z"] = 9.0
        f[1]["x"] = 10.0
        f[1]["y"] = 11.0
        f[1]["z"] = 12.0
        return f

    @g_vector_array.setter
    def g_vector_array(self, value):
        assert (value[0]["x"] == 1.0)
        assert (value[0]["y"] == 2.0)
        assert (value[0]["z"] == 3.0)
        assert (value[1]["x"] == 4.0)
        assert (value[1]["y"] == 5.0)
        assert (value[1]["z"] == 6.0)

    @property
    def h_quaternion(self):
        j = np.zeros((1,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyQuaternion"))
        j[0]["w"] = 0.707
        j[0]["x"] = 0.0
        j[0]["y"] = 0.707
        j[0]["z"] = 0.0
        return j

    @h_quaternion.setter
    def h_quaternion(self, value):
        assert (value[0]["w"] == 1.0)
        assert (value[0]["x"] == 0.0)
        assert (value[0]["y"] == 0.0)
        assert (value[0]["z"] == 0.0)

    @property
    def i_pose(self):
        l = np.zeros((1,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyPose"))
        l[0]["orientation"]["w"] = 0.707
        l[0]["orientation"]["x"] = 0.0
        l[0]["orientation"]["y"] = 0.707
        l[0]["orientation"]["z"] = 0.0
        l[0]["position"]["x"] = 4.0
        l[0]["position"]["y"] = 5.0
        l[0]["position"]["z"] = 6.0
        return l

    @i_pose.setter
    def i_pose(self, value):
        assert (value[0]["orientation"]["w"] == 1.0)
        assert (value[0]["orientation"]["x"] == 0.0)
        assert (value[0]["orientation"]["y"] == 0.0)
        assert (value[0]["orientation"]["z"] == 0.0)
        assert (value[0]["position"]["x"] == 1.0)
        assert (value[0]["position"]["y"] == 2.0)
        assert (value[0]["position"]["z"] == 3.0)

    @property
    def j_pod(self):
        a = np.zeros((1,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyPod"))
        a[0]["a"] = 0.791
        a[0]["b"] = 1077
        a[0]["c"] = [61, 52, 33, 24]
        a[0]["d"]["len"] = 2
        a[0]["d"]["array"][0:2] = [1, 2]
        a[0]["e"] = [[7, 8, 9], [10, 11, 12]]
        a[0]["f"]["x"] = 20.1
        a[0]["f"]["y"] = 20.2
        a[0]["f"]["z"] = 20.3
        return a

    @j_pod.setter
    def j_pod(self, value):
        assert (value[0]["a"] == 0.928)
        assert (value[0]["b"] == 8374)
        np.testing.assert_array_equal(value[0]["c"], [8, 9, 10, 11])
        assert (value[0]["d"]["len"] == 1)
        np.testing.assert_array_equal(value[0]["d"]["array"][0:1], [17])
        np.testing.assert_array_equal(value[0]["e"], [[1, 2, 3], [4, 5, 6]])
        assert (value[0]["f"]["x"] == 10.1)
        assert (value[0]["f"]["y"] == 10.2)
        assert (value[0]["f"]["z"] == 10.3)

    @property
    def j_pod_array(self):
        d = np.zeros((2,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyPod"))
        d[0]["a"] = 0.791
        d[0]["b"] = 1077
        d[0]["c"] = [61, 52, 33, 24]
        d[0]["d"]["len"] = 2
        d[0]["d"]["array"][0:2] = [1, 2]
        d[0]["e"] = [[7, 8, 9], [10, 11, 12]]
        d[0]["f"]["x"] = 20.1
        d[0]["f"]["y"] = 20.2
        d[0]["f"]["z"] = 20.3
        d[1]["a"] = 0.03
        d[1]["b"] = 693
        d[1]["c"] = [5, 6, 7, 8]
        d[1]["d"]["len"] = 1
        d[1]["d"]["array"][0] = 3
        d[1]["e"] = [[19, 20, 21], [22, 23, 24]]
        d[1]["f"]["x"] = 40.1
        d[1]["f"]["y"] = 40.2
        d[1]["f"]["z"] = 40.3
        return d

    @j_pod_array.setter
    def j_pod_array(self, value):
        assert (value[0]["a"] == 0.928)
        assert (value[0]["b"] == 8374)
        np.testing.assert_array_equal(value[0]["c"], [8, 9, 10, 11])
        assert (value[0]["d"]["len"] == 1)
        np.testing.assert_array_equal(value[0]["d"]["array"][0:1], [17])
        np.testing.assert_array_equal(value[0]["e"], [[1, 2, 3], [4, 5, 6]])
        assert (value[0]["f"]["x"] == 10.1)
        assert (value[0]["f"]["y"] == 10.2)
        assert (value[0]["f"]["z"] == 10.3)
        assert (value[1]["a"] == 0.67)
        assert (value[1]["b"] == 123)
        np.testing.assert_array_equal(value[1]["c"], [1, 2, 3, 4])
        assert (value[1]["d"]["len"] == 2)
        np.testing.assert_array_equal(value[1]["d"]["array"][0:2], [5, 6])
        np.testing.assert_array_equal(value[1]["e"], [[13, 14, 15], [16, 17, 18]])
        assert (value[1]["f"]["x"] == 30.1)
        assert (value[1]["f"]["y"] == 30.2)
        assert (value[1]["f"]["z"] == 30.3)

    # Containers
    @property
    def l_double_map(self):
        return {1: 5.1, 5: 6.2}

    @l_double_map.setter
    def l_double_map(self, value):
        assert (len(value) == 3)
        assert (value[1] == 1.1)
        assert (value[2] == 2.2)
        assert (value[5] == 3.3)

    @property
    def l_double_array_map(self):
        return {
            "key3": np.array([5.1, 5.2, 5.3], dtype=np.float64),
            "key4": np.array([6.1, 6.2, 6.3], dtype=np.float64)
        }

    @l_double_array_map.setter
    def l_double_array_map(self, value):
        assert (len(value) == 2)
        np.testing.assert_array_equal(value["key1"], [1.1, 1.2, 1.3])
        np.testing.assert_array_equal(value["key2"], [2.1, 2.2, 2.3])

    @property
    def m_string_list(self):
        return ["string 4", "string 5"]

    @m_string_list.setter
    def m_string_list(self, value):
        assert (len(value) == 3)
        assert (value[0] == "string 1")
        assert (value[1] == "string 2")
        assert (value[2] == "string 3")

    @property
    def m_string_map_int32(self):
        return {
            87: "string 4",
            1: "string 5"
        }

    @m_string_map_int32.setter
    def m_string_map_int32(self, value):
        assert (len(value) == 3)
        assert (value[12] == "string 1")
        assert (value[100] == "string 2")
        assert (value[17] == "string 3")

    @property
    def m_string_map_string(self):
        return {
            "key3": "string 3",
            "key4": "string 4",
            "key5": "string 5"
        }

    @m_string_map_string.setter
    def m_string_map_string(self, value):
        assert (len(value) == 2)
        assert (value["key1"] == "string 1")
        assert (value["key2"] == "string 2")

    @property
    def n_vector_map(self):
        r = np.zeros((1,), dtype=RRN.GetNamedArrayDType("experimental.value_types.MyVector3"))
        r[0]["x"] = 4.0
        r[0]["y"] = 5.0
        r[0]["z"] = 6.0
        return {1: r}

    @n_vector_map.setter
    def n_vector_map(self, value):
        assert (len(value) == 1)
        assert (value[1][0]["x"] == 1.0)
        assert (value[1][0]["y"] == 2.0)
        assert (value[1][0]["z"] == 3.0)

    @property
    def o_struct_list(self):
        s = RRN.NewStructure("experimental.value_types.MyStructure")
        fill_MyStructure(s)
        return [s]

    @o_struct_list.setter
    def o_struct_list(self, value):
        assert (len(value) == 1)
        assert_MyStructure(value[0])

    @property
    def p_varvalue_double_array(self):
        return RR.VarValue(np.array([1.4, 2.5, 3.6], dtype=np.float64), "double[]")

    @p_varvalue_double_array.setter
    def p_varvalue_double_array(self, value):
        np.testing.assert_array_equal(value.data, [1.1, 2.2, 3.3])
        assert value.datatype == "double[]"

    @property
    def q_varvalue_string(self):
        return RR.VarValue("varvalue string from service", "string")

    @q_varvalue_string.setter
    def q_varvalue_string(self, value):
        assert value.data == "varvalue string from client"
        assert value.datatype == "string"

    @property
    def r_varvalue_struct(self):
        my_structure_type = RRN.GetStructureType("experimental.value_types.MyStructure")
        s = my_structure_type()
        fill_MyStructure(s)
        return RR.VarValue(s, "experimental.value_types.MyStructure")

    @r_varvalue_struct.setter
    def r_varvalue_struct(self, value):
        assert_MyStructure(value.data)
        assert value.datatype == "experimental.value_types.MyStructure"

    @property
    def s_varvalue_map2(self):
        return {
            "key3": RR.VarValue(np.array([4, 5], dtype=np.int32), "int32[]"),
            "key4": RR.VarValue("string 4", "string")
        }

    @s_varvalue_map2.setter
    def s_varvalue_map2(self, value):
        assert len(value) == 2
        np.testing.assert_array_equal(value["key1"].data, [2, 3])
        assert value["key1"].datatype == "int32[]"
        assert value["key2"].data == "string 2"
        assert value["key2"].datatype == "string"

    @property
    def t_struct_null(self):
        return None

    @t_struct_null.setter
    def t_struct_null(self, value):
        assert value is None


def fill_MyStructure(s, i=0):
    s.a = 52 + i
    s.b = np.array([110, 120], dtype=np.uint32)
    s.c = "String from structure service"
    s.d = [
        "string c",
        "string d",
        "string e"
    ]


def assert_MyStructure(u, i=0):
    assert u is not None
    assert (u.a == 5 + i)
    np.testing.assert_array_equal(u.b, np.array([10, 20], dtype=np.uint32))
    assert (u.c == "String from structure client")
    assert (len(u.d) == 2)
    assert (u.d[0] == "string a")
    assert (u.d[1] == "string b")


def main():
    with RR.ServerNodeSetup("experimental.value_types", 53223) as node_setup:

        # Register the service type
        RRN.RegisterServiceTypeFromFile("../robdef/experimental.value_types.robdef")

        # Create the object
        obj = ValueTypesExample_impl()

        # Register the service
        RRN.RegisterService("values_example", "experimental.value_types.ValueTypesExample", obj)

        # Wait for the user to quit
        input("Server started, press enter to quit...")


if __name__ == '__main__':
    main()



from github import Github
import argparse
import os
from pathlib import Path
import shutil
import urllib
import subprocess
import re


def main():

    parser = argparse.ArgumentParser(
        description="Build Matlab toolbox and upload to release")
    parser.add_argument("--repository", type=str,
                        default=None, help="Repository to build for")
    parser.add_argument("--tag-name", type=str,
                        default=None, help="Release tag")
    parser.add_argument(
        "--token-file", type=argparse.FileType("r"), help="Load token from file")
    parser.add_argument("--matlab-dir", type=str,
                        default="/usr/local/MATLAB/R2020b", help="Matlab directory")
    parser.add_argument("--no-download", action="store_true")

    args = parser.parse_args()

    matlab_dir = Path(args.matlab_dir)
    matlab_exe = matlab_dir.joinpath("bin/matlab")
    assert matlab_exe.is_file()

    workdir = Path("build-matlab-toolbox").absolute()

    if not args.no_download:
        if workdir.is_dir():
            shutil.rmtree(workdir)

        workdir.mkdir()

    repo_path = args.repository

    if repo_path is None:
        repo_path = os.environ.get(
            "GITHUB_REPOSITORY", "robotraconteur/robotraconteur")

    token = None

    if args.token_file:
        token = args.token_file.read().strip()

    if token is None:
        token = os.environ.get("BOT_GITHUB_TOKEN", None)

    github = Github(token)
    repo = github.get_repo(repo_path)
    releases = repo.get_releases()

    release = None

    tag_name = args.tag_name
    input_tag_name = os.environ.get("INPUT_TAG_NAME", None)
    if input_tag_name is not None and len(input_tag_name) > 0:
        tag_name = input_tag_name
        print(f"Detected tag_name={tag_name} from input")

    for r in releases:
        if tag_name is None:
            if r.draft:
                release = r
                tag_name = r.tag_name
                break
        elif r.tag_name == tag_name:
            release = r
            break

    assert release, "Could not find release"

    print(release.tag_name)

    release_assets = release.get_assets()

    release_assets2 = filter(lambda a: "MATLAB" in a.name, release_assets)
    for a in release_assets2:
        print(a.name)
        print(a.browser_download_url)
        if not args.no_download:
            download_asset(a.url, a.name, token)

    build_dir = Path("build-matlab-toolbox/build")
    matlab_dir = build_dir.joinpath("matlab")
    matlab_dir.mkdir(exist_ok=True, parents=True)

    subprocess.check_call(
        "tar xf ../../RobotRaconteur-*-MATLAB-glnxa64.tar.gz --strip-components 1", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "tar xf ../../RobotRaconteur-*-MATLAB-maci64.tar.gz --strip-components 1 --wildcards RobotRaconteur-*-MATLAB-maci64/RobotRaconteurMex.*", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "tar xf ../../RobotRaconteur-*-MATLAB-maca64.tar.gz --strip-components 1 --wildcards RobotRaconteur-*-MATLAB-maca64/RobotRaconteurMex.*", shell=True, cwd=matlab_dir)
    subprocess.check_call(
        "unzip -j ../../RobotRaconteur-*-MATLAB-win64.zip RobotRaconteur-*-MATLAB-win64/RobotRaconteurMex.mexw64", shell=True, cwd=matlab_dir)

    semver_regex = r"^v((?:0|[1-9]\d*)\.(?:0|[1-9]\d*)\.(?:0|[1-9]\d*))(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"

    print(tag_name)
    semver_match = re.match(semver_regex, tag_name)

    toolbox_ver = semver_match.group(1)

    with open("robotraconteur/packaging/matlab/RobotRaconteurMatlab.prj.in") as f1:
        s1 = f1.read()
    with open(build_dir.joinpath("RobotRaconteurMatlab.prj"), "w") as f2:
        f2.write(s1.replace("$(rr_version)", toolbox_ver))

    shutil.copy("robotraconteur/packaging/matlab/logo-icon.png", build_dir)

    subprocess.check_call(
        f"{matlab_exe} -nodesktop -batch \"matlab.addons.toolbox.packageToolbox('RobotRaconteurMatlab.prj','RobotRaconteurMatlab'); exit\"", shell=True, cwd=build_dir)

    shutil.move(f"{build_dir}/RobotRaconteurMatlab.mltbx",
                f"{build_dir}/RobotRaconteurMatlab-{toolbox_ver}.mltbx")

    release.upload_asset(f"{build_dir}/RobotRaconteurMatlab-{toolbox_ver}.mltbx",
                         content_type="application/octet-stream")


def download_asset(asset_url, name, token):
    print("Retrieving release asset %s from url: %s" % (name, asset_url))

    headers = {"Authorization": f"token {token}",
               "Accept": "application/octet-stream"}
    req = urllib.request.Request(asset_url, headers=headers)
    response = urllib.request.urlopen(req)
    headers = response.getheaders()
    with open(f"build-matlab-toolbox/{name}", 'wb') as f:
        f.write(response.read())


if __name__ == "__main__":
    main()



# subscribe_filter.py - Subscribe to services by type using a filter

from RobotRaconteur.Client import *
import time


def subscription_predicate(service_info2):
    # This function is called for each discovered service
    # Return True to connect to the service, False to ignore
    # The service_info2 parameter is a ServiceInfo2 object
    # containing information about the discovered service
    return True


# Populate a subscription filter. Note that all empty fields are considered "wildcards" that match any value
f = RR.ServiceSubscriptionFilter()
# Only connect to services named "reynard"
f.ServiceNames = ['reynard']
# Only use rr+tcp transport
f.TransportSchemes = ['rr+tcp', 'rr+local']
# The ServiceSubscriptionFilter has powerful attribute matching capabilities
# The use is intended for advanced use cases
# f.Attributes = <Advanced attribute matching>
# f.AttributesMatchOperation = ServiceSubscriptionFilterAttributeGroupOperation_AND
# Set a "predicate" function that is called for each discovered service
f.Predicate = subscription_predicate
# Set the maximum number of clients to connect. Default is -1 (no limit)
f.MaxConnections = 5

# Create a ServiceSubscriptionFilterNode for node matching
n = RR.ServiceSubscriptionFilterNode()
# Only connect to nodes named "experimental.reynard_the_robot"
n.NodeName = "experimental.reynard_the_robot"
# Only connect to nodes with NodeID as listed. This will be different for every instance of the node
# n.NodeID = RR.NodeID('12345678-1234-1234-1234-123456789abc')
# Username and credentials can also be set for services that require authentication.
# This should only be used when NodeID is also specified so the credentials are not sent to the wrong node
# n.Username = "username"
# n.Credentials = {"password", RR.VarValue("my_password", "string")}
f.Nodes.append(n)

# Subscribe to a service by type using the filter
root_object_type = "experimental.reynard_the_robot.Reynard"
sub = RRN.SubscribeServiceByType(root_object_type, f)
# Give a few seconds for the service to be connected
time.sleep(6)

# Run a function on all connected services. Could also use GetDefaultClientWait() to
# get the first connected client and run functions on that client
# without waiting for all clients to connect
connected_clients = sub.GetConnectedClients()
for k, c in connected_clients.items():
    c.say("Hello from subscribe_filter!")



from __future__ import absolute_import

import RobotRaconteur as RR


class asynctestroot_impl(object):

    def RRServiceObjectInit(self, ctx, service_path):
        self._node = ctx.GetNode()

    def async_get_d1(self, handler):
        self._node.PostToThreadPool(lambda: handler(8.5515, None))

    def async_set_d1(self, value, handler):
        if value != 3.0819:
            self._node.PostToThreadPool(
                lambda: handler(RR.InvalidArgumentException()))
            return
        self._node.PostToThreadPool(lambda: handler(None))

    def async_get_err(self, handler):
        self._node.PostToThreadPool(lambda: handler(
            None, RR.InvalidArgumentException("Test message 1")))

    def async_set_err(self, val, handler):
        self._node.PostToThreadPool(
            lambda: handler(RR.InvalidOperationException("")))

    def async_f1(self, handler):
        self._node.PostToThreadPool(lambda: handler(None))

    def async_f2(self, a, handler):
        if a != 247:
            self._node.PostToThreadPool(
                lambda: handler(RR.InvalidArgumentException()))
            return
        self._node.PostToThreadPool(lambda: handler(None))

    def async_f3(self, a, b, handler):
        res = int(a + b)
        self._node.PostToThreadPool(lambda: handler(res, None))

    def async_err_func(self, handler):
        self._node.PostToThreadPool(
            lambda: handler(RR.InvalidOperationException()))

    def async_err_func2(self, handler):
        asynctestexp = self._node.GetExceptionType(
            "com.robotraconteur.testing.TestService5.asynctestexp")
        self._node.PostToThreadPool(lambda: handler(None, asynctestexp("")))



# container_value_types.py - Example of using container value types (maps and lists)

from RobotRaconteur.Client import *
import numpy as np

c = RRN.ConnectService('rr+tcp://localhost:53223?service=values_example')

# property double{int32} l_double_map
# int32 keyed map of scalar doubles
c.l_double_map = {
    1: 1.1,
    2: 2.2,
    5: 3.3
}

b = c.l_double_map
assert (len(b) == 2)
assert (b[1] == 5.1)
assert (b[5] == 6.2)

# property double[]{string} l_double_array_map
# string keyed map of double arrays
c.l_double_array_map = {
    "key1": np.array([1.1, 1.2, 1.3], dtype=np.float64),
    "key2": np.array([2.1, 2.2, 2.3], dtype=np.float64)
}

d = c.l_double_array_map
assert (len(d) == 2)
np.testing.assert_array_equal(d["key3"], [5.1, 5.2, 5.3])
np.testing.assert_array_equal(d["key4"], [6.1, 6.2, 6.3])

# property string{list} m_string_list
# list of strings
c.m_string_list = ["string 1", "string 2", "string 3"]

a = c.m_string_list
assert (len(a) == 2)
assert (a[0] == "string 4")
assert (a[1] == "string 5")


# property string{int32} m_string_map_int32
# int32 keyed map of strings
c.m_string_map_int32 = {
    12: "string 1",
    100: "string 2",
    17: "string 3"
}

e = c.m_string_map_int32
assert (len(e) == 2)
assert (e[87] == "string 4")
assert (e[1] == "string 5")

# property string{string} m_string_map_string
# string keyed map of strings
c.m_string_map_string = {
    "key1": "string 1",
    "key2": "string 2"
}

f = c.m_string_map_string
assert (len(f) == 3)
assert (f["key3"] == "string 3")
assert (f["key4"] == "string 4")
assert (f["key5"] == "string 5")

# property MyVector3{int32} n_vector_map
# int32 keyed map of MyVector3
my_vector3_dtype = RRN.GetNamedArrayDType("experimental.value_types.MyVector3", obj=c)
e = np.zeros((1,), dtype=my_vector3_dtype)
e[0]["x"] = 1.0
e[0]["y"] = 2.0
e[0]["z"] = 3.0
c.n_vector_map = {
    1: e
}

g = c.n_vector_map
assert (len(g) == 1)
g1 = g[1]
assert (g1[0]["x"] == 4.0)
assert (g1[0]["y"] == 5.0)
assert (g1[0]["z"] == 6.0)

# property MyStructure{list} o_struct_list
# list of MyStructure
my_structure_type = RRN.GetStructureType("experimental.value_types.MyStructure", obj=c)
h = my_structure_type()
h.a = 5
h.b = np.array([10, 20], dtype=np.uint32)
h.c = "String from structure client"
h.d = [
    "string a",
    "string b"
]
c.o_struct_list = [h]

i = c.o_struct_list
assert (len(i) == 1)
i0 = i[0]
assert (i0.a == 52)
np.testing.assert_array_equal(i[0].b, np.array([110, 120], dtype=np.uint32))
assert (i0.c == "String from structure service")
assert (len(i[0].d) == 3)
assert (i0.d[0] == "string c")
assert (i0.d[1] == "string d")
assert (i0.d[2] == "string e")



from RobotRaconteur.Client import *
from .ServiceTestClient3 import ServiceTestClient3


def _member_test(fn, test_server_node_config):
    service_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService3")
    c = ServiceTestClient3()
    c.ConnectService(service_url)
    try:
        fn(c)
    finally:
        c.DisconnectService()


def test_properties3(test_server_node_config):
    _member_test(lambda c: c.TestProperties(), test_server_node_config)


def test_functions3(test_server_node_config):
    _member_test(lambda c: c.TestFunctions(), test_server_node_config)



import RobotRaconteur as RR
import pytest


_tls_test_service_def = """
service experimental.tls_test

object testobj
    function double add_two_numbers(double a, double b)
end object
"""


class _testobj_impl():

    def add_two_numbers(self, a, b):
        return a + b


def test_robotraconteur_tcp_tls():
    node1 = RR.RobotRaconteurNode()
    node1.Init()
    node2 = RR.RobotRaconteurNode()
    node2.Init()
    node3 = RR.RobotRaconteurNode()
    node3.Init()
    node4 = RR.RobotRaconteurNode()
    node4.Init()

    tls_client_flags = RR.RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT \
        | RR.RobotRaconteurNodeSetupFlags_LOAD_TLS_CERT
    client_override = RR.RobotRaconteurNodeSetupFlags_CLIENT_DEFAULT_ALLOWED_OVERRIDE

    node1_setup = RR.SecureServerNodeSetup("testprog_tls1", 0, node=node1)
    node2_setup = RR.ClientNodeSetup(node=node2)
    node3_setup = RR.RobotRaconteurNodeSetup(node_name="testprog_tls2", tcp_port=0, flags=tls_client_flags,
                                             allowed_overrides=client_override, node=node3)
    node4_setup = RR.RobotRaconteurNodeSetup(node_name="testprog_tls3", tcp_port=0, flags=tls_client_flags,
                                             allowed_overrides=client_override, node=node4)

    node1.RegisterServiceType(_tls_test_service_def)

    obj = _testobj_impl()
    obj2 = _testobj_impl()
    ctx = node1.RegisterService(
        "test_service", "experimental.tls_test.testobj", obj)

    authdata = "testuser1 0b91dec4fe98266a03b136b59219d0d6 objectlock " + str(node3.NodeID) + "\n" \
        + "testuser2 841c4221c2e7e0cefbc0392a35222512 objectlock " + str(node4.NodeID) + "\n" \
        + "testsuperuser 503ed776c50169f681ad7bbc14198b68 objectlock,objectlockoverride " + \
        str(node3.NodeID) + "\n"
    p = RR.PasswordFileUserAuthenticator(authdata, True)
    policies = {"requirevaliduser": "true", "allowobjectlock": "true"}
    s = RR.ServiceSecurityPolicy(p, policies)
    ctx_secure = node1.RegisterService(
        "test_service_secure", "experimental.tls_test.testobj", obj2, s)

    server_tcp_port = node1_setup.tcp_transport.GetListenPort()

    url = f"rrs+tcp://localhost:{server_tcp_port}?service=test_service"
    url_unsecure = f"rr+tcp://localhost:{server_tcp_port}?service=test_service"
    url_ws = f"rrs+ws://localhost:{server_tcp_port}?service=test_service"
    url_unsecure_ws = f"rr+ws://localhost:{server_tcp_port}?service=test_service"
    url_secure = f"rrs+tcp://localhost:{server_tcp_port}?service=test_service_secure"

    with node1_setup, node2_setup, node3_setup, node4_setup:
        c1 = node2.ConnectService(url)
        assert c1.add_two_numbers(5, 6) == 11
        node2.DisconnectService(c1)

        # Make sure that unencrypted connections are dropped
        with pytest.raises(RR.ConnectionException):
            c2 = node2.ConnectService(url_unsecure)

        c1 = node2.ConnectService(url_ws)
        assert c1.add_two_numbers(5, 6) == 11
        node2.DisconnectService(c1)

        # Make sure that unencrypted connections are dropped
        with pytest.raises(RR.ConnectionException):
            c2 = node2.ConnectService(url_unsecure_ws)
        user1 = "testuser1"
        user2 = "testuser2"
        cred1 = {"password": RR.VarValue("testpass1", "string")}
        cred2 = {"password": RR.VarValue("testpass2", "string")}

        with pytest.raises(RR.AuthenticationException):
            node2.ConnectService(url_secure, user1, cred1)

        c3 = node3.ConnectService(url_secure, user1, cred1)
        assert c3.add_two_numbers(5, 7) == 12
        node3.DisconnectService(c3)

        with pytest.raises(RR.AuthenticationException):
            node4.ConnectService(url_secure, user1, cred1)

        c4 = node4.ConnectService(url_secure, user2, cred2)
        assert c4.add_two_numbers(9, 7) == 16
        node4.DisconnectService(c4)



# For dist.yml github action

import shutil
import pathlib

asset_dirs = ["artifacts/main"]

pathlib.Path("distfiles").mkdir(exist_ok=True)

for asset_dir in asset_dirs:
    for fname in list(pathlib.Path(asset_dir).glob('**/wheels/*.whl')):
        print(fname)
        dest = pathlib.Path(fname)
        shutil.copy(str(fname), "distfiles/" + dest.name)



# Simple example Robot Raconteur standard camera client
# This program will show a live streamed image from
# the camera.

from RobotRaconteur.Client import *
from RobotRaconteurCompanion.Util.ImageUtil import ImageUtil

import cv2
import sys
import traceback
import argparse
import numpy as np
import traceback

current_frame = None
image_util = None

# This function is called when a new pipe packet arrives


def new_frame(pipe_ep):
    global current_frame

    # Loop to get the newest frame
    while (pipe_ep.Available > 0):
        # Receive the packet
        image = pipe_ep.ReceivePacket()
        # Convert the packet to an image and set the global variable
        current_frame = image_util.image_to_array(image)
        return


def main():

    # URL for connecting to the camera. By default connects to Gazebo simulated camera
    url = 'rr+tcp://localhost:59823?service=camera'
    if (len(sys.argv) >= 2):
        url = sys.argv[1]

    # Connect to the camera
    cam = RRN.ConnectService(url)

    # Create an ImageUtil object to help with image conversion
    global image_util
    image_util = ImageUtil(RRN, cam)

    # Connect the pipe frame_stream to get the PipeEndpoint p

    p = cam.frame_stream.Connect(-1)

    # Set the callback for when a new pipe packet is received to the
    # new_frame function
    p.PacketReceivedEvent += new_frame

    # Start the camera streaming, ignore error if it is already streaming
    try:
        cam.start_streaming()
    except:
        traceback.print_exc()
        pass

    cv2.namedWindow("Image")

    while True:
        # Display the image
        if (not current_frame is None):

            cv2.imshow("Image", current_frame)
        if cv2.waitKey(50) != -1:
            break
    cv2.destroyAllWindows()

    p.Close()
    cam.stop_streaming()


if __name__ == '__main__':
    main()



import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s
import numpy as np


class AuthenticationExample_impl:
    def say(self, message):
        print(f"Message: {message}")


def main():
    with RR.ServerNodeSetup("experimental.authentication_example", 53226) as node_setup:

        # Use reynard the robot robdef but only implement "say"
        RRN.RegisterServiceTypeFromFile("../../../reynard_the_robot/robdef/experimental.reynard_the_robot.robdef")

        # Create the object
        obj = AuthenticationExample_impl()

        # Set the authentication configuration
        authdata = "testuser1 0b91dec4fe98266a03b136b59219d0d6 objectlock\ntestuser2 841c4221c2e7e0cefbc0392a35222512 objectlock\ntestsuperuser 503ed776c50169f681ad7bbc14198b68 objectlock,objectlockoverride"
        p = RR.PasswordFileUserAuthenticator(authdata)
        policies = {"requirevaliduser": "true", "allowobjectlock": "true"}
        s = RR.ServiceSecurityPolicy(p, policies)

        # Register the service
        RRN.RegisterService("authentication_example", "experimental.reynard_the_robot.Reynard", obj, s)

        # Wait for the user to quit
        input("Server started, press enter to quit...")


if __name__ == '__main__':
    main()



# struct_value_type.py - Simple example of using struct value types

from RobotRaconteur.Client import *
import numpy as np

c = RRN.ConnectService('rr+tcp://localhost:53223?service=values_example')

# Example using MyStructure type. Also see example
# using MyStructure for a more complex example.

# Retrieve the structure type. obj is only used
# for clients. It is omitted for services.
my_structure_type = RRN.GetStructureType("experimental.value_types.MyStructure", obj=c)

# Create and populate a MyStructure
s = my_structure_type()
# field double a
s.a = 5
# field uint32[2] b
s.b = np.array([10, 20], dtype=np.uint32)
# field string c
s.c = "String from structure client"
# field string{list} d
s.d = [
    "string a",
    "string b"
]

# Set the property using the structure
# property MyStructure k_struct
c.k_struct = s

# Retrieve the structure from the service
u = c.k_struct

# Structures can be None
assert u is not None

# field double a
assert (u.a == 52)
# field uint32[2] b
np.testing.assert_array_equal(u.b, np.array([110, 120], dtype=np.uint32))
# field string c
assert (u.c == "String from structure service")
# field string{list} d
assert (len(u.d) == 3)
assert (u.d[0] == "string c")
assert (u.d[1] == "string d")
assert (u.d[2] == "string e")



from RobotRaconteur.Client import *
import sys
import time


def main():
    rrtype = sys.argv[1]
    transports = sys.argv[2].split(",")

    time.sleep(6)

    ret = RRN.FindServiceByType(rrtype, transports)

    for r in ret:
        print_ServiceInfo2(r)

    def printret(ret):
        for r in ret:
            print_ServiceInfo2(r)
    RRN.AsyncFindServiceByType(rrtype, transports, printret, 1)

    time.sleep(2)


def print_ServiceInfo2(s):
    print("Name: " + s.Name)
    print("RootObjectType: " + s.RootObjectType)
    print("RootObjectImplements: " + str(s.RootObjectImplements))
    print("ConnectionURL: " + str(s.ConnectionURL))
    print("Attributes: " + str(s.Attributes))
    print("NodeID: " + str(s.NodeID))
    print("NodeName: " + s.NodeName)
    print("")


if __name__ == "__main__":
    main()



# reynard_the_robot_client_pyodide.py - Reynard the Robot client using Pyodide

from RobotRaconteur.Client import *
import js
import asyncio
import traceback


async def main():
    status_div = js.document.getElementById("status")
    status_div.innerHTML = "Connecting..."
    error_log = js.document.getElementById("error_log")
    try:

        RRN.SetLogLevel(RR.LogLevel_Debug)

        # All Robot Raconteur calls must use asynchronous variants
        # in Pyodide because browsers are single threaded

        # Connect to Reynard the Robot service using websocket
        url = 'rr+tcp://localhost:29200?service=reynard'
        c = await RRN.AsyncConnectService(url, None, None, None, None)

        received_messages_div = js.document.getElementById("received_messages")

        def new_message(msg):
            received_messages_div.innerHTML += f"{msg}<br>"

        # Connect a callback function to listen for new messages
        c.new_message += new_message

        # Handle when the send_message button is clicked
        send_button = js.document.getElementById("send_message")

        async def do_send_message():
            try:
                message = js.document.getElementById("message").value
                await c.async_say(message, None)
            except Exception as e:
                error_log.innerHTML += f"Error: {traceback.format_exc()}" + "<br>"
        send_button.onclick = lambda _: asyncio.create_task(do_send_message())

        teleport_button = js.document.getElementById("teleport")

        async def do_teleport():
            try:
                x = float(js.document.getElementById("teleport_x").value) * 1e-3
                y = float(js.document.getElementById("teleport_y").value) * 1e-3
                await c.async_teleport(x, y, None)
            except Exception as e:
                error_log.innerHTML += f"Error: {traceback.format_exc()}" + "<br>"
        teleport_button.onclick = lambda _: asyncio.create_task(do_teleport())

        status_div.innerHTML = "Connected"

        # Run loop to update Reynard position
        while True:
            try:
                # Read the current state using a wire "peek". Can also "connect" to receive streaming updates.
                state, _ = await c.state.AsyncPeekInValue(None)
                js.document.getElementById(
                    "reynard_position").innerHTML = f"x: {state.robot_position[0]}, y: {state.robot_position[1]}"
            except:
                status_div.innerHTML = "Error"
                error_log.innerHTML += f"Error: {traceback.format_exc()}" + "<br>"
            await asyncio.sleep(0.1)

    except:
        status_div.innerHTML = "Error"
        error_log.innerHTML += f"Error: {traceback.format_exc()}" + "<br>"

asyncio.create_task(main())



import argparse
import os
from pathlib import Path
import shutil
import urllib
import subprocess
import re


def main():

    with open("artifacts/main/out-matlab/rrversion.txt", "r") as f:
        tag_name = f.read().strip()

    print(f"tag_name: {tag_name}")

    build_dir = Path("build-matlab-toolbox/build")
    matlab_dir = build_dir.joinpath("matlab")

    semver_regex = r"^((?:0|[1-9]\d*)\.(?:0|[1-9]\d*)\.(?:0|[1-9]\d*))(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"

    semver_match = re.match(semver_regex, tag_name)

    toolbox_ver = semver_match.group(1)

    shutil.move(f"{build_dir}/RobotRaconteurMatlab.mltbx",
                f"{build_dir}/RobotRaconteurMatlab-{toolbox_ver}.mltbx")


if __name__ == "__main__":
    main()



import RobotRaconteur as RR
import pytest
import time
import threading

_wire_sub_test_service_def = """
service experimental.wire_sub_test

object testobj
    wire double testwire1 [readonly]
    wire double testwire2 [writeonly]
    objref testobj2 subobj
end

object testobj2
    wire double testwire3 [readonly]
end
"""


class _testobj_impl:
    def __init__(self):
        self.subobj = _testobj2_impl()

    def get_subobj(self):
        return self.subobj, "experimental.wire_sub_test.testobj2"


class _testobj2_impl:
    def __init__(self):
        pass


intra_server_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_INTRA_TRANSPORT_START_SERVER \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_ANNOUNCE \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE

intra_client_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE


class _testservice_impl():

    def __init__(self, nodename, nodeid):
        self.obj = _testobj_impl()
        self._node = RR.RobotRaconteurNode()
        self._node.SetNodeID(nodeid)
        self._node.Init()

        self._node.RegisterServiceType(_wire_sub_test_service_def)
        self._node.RegisterService(
            "test_service", "experimental.wire_sub_test.testobj", self.obj)

        self._node_setup = RR.RobotRaconteurNodeSetup(
            nodename, 0, flags=intra_server_flags, node=self._node)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._node_setup.ReleaseNode()
        self._node.Shutdown()
        self._node_setup = None
        self._node = None


def test_wire_subscription():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    server1 = _testservice_impl("server1", test_servers["server1"])

    with client_node_setup, server1:

        value_changed_count = [0]

        def value_changed(wire, value, time):
            value_changed_count[0] += 1

        sub = client_node.SubscribeServiceByType(
            "experimental.wire_sub_test.testobj")
        wire_sub = sub.SubscribeWire("testwire1")
        wire_sub.WireValueChanged += value_changed
        sub.GetDefaultClientWait(5)

        for _ in range(10):
            if wire_sub.ActiveWireConnectionCount > 0:
                break
            time.sleep(0.01)
        else:
            assert False

        with pytest.raises(RR.ValueNotSetException):
            _ = wire_sub.InValue
        assert not wire_sub.TryGetInValue()[0]

        server1.obj.testwire1.OutValue = 5.0
        time.sleep(0.001)
        server1.obj.testwire1.OutValue = 5.0

        wire_sub.WaitInValueValid(1)

        assert wire_sub.InValue == 5.0
        assert wire_sub.TryGetInValue()[1] == 5.0

        for _ in range(10):
            if value_changed_count[0] > 0:
                break
            time.sleep(0.01)
        else:
            assert False

        wire_sub.InValueLifespan = 0.1
        assert wire_sub.InValueLifespan == 0.1
        time.sleep(0.2)
        assert not wire_sub.TryGetInValue()[0]

        wire_sub.InValueLifespan = 10
        server1.obj.testwire1.OutValue = 6.0
        time.sleep(0.05)
        assert wire_sub.InValue == 6.0

        wire_sub2 = sub.SubscribeWire("testwire3", "*.subobj")

        time.sleep(0.05)

        assert wire_sub2.ActiveWireConnectionCount > 0

        time.sleep(0.2)

        server1.obj.subobj.testwire3.OutValue = 12.345

        wire_sub2.WaitInValueValid(1)
        assert wire_sub2.InValue == 12.345

        wire_sub2.Close()

        wire_sub3 = sub.SubscribeWire("testwire2")
        time.sleep(0.05)
        assert wire_sub3.ActiveWireConnectionCount > 0
        wire_sub3.SetOutValueAll(17.0)

        time.sleep(0.05)
        assert server1.obj.testwire2.InValue[0] == 17.0
        res, val, _, _ = server1.obj.testwire2.TryGetInValue()
        assert res and val == 17.0
        server1.obj.testwire2.InValueLifespan = 0.1
        time.sleep(0.2)
        assert not server1.obj.testwire2.TryGetInValue()[0]



from pathlib import Path
import hashlib
import urllib.request
import sys
import re
import os
import shutil

ref = os.environ["GITHUB_SHA"]
ver = ref

ref1 = os.environ.get("GITHUB_REF", None)

if ref1 is not None:
    ref_match = re.match(r"refs/tags/v(\d+\.\d+\.\d+.*)", ref1)
    if ref_match is not None:
        ref = ref1
        ver = ref_match.group(1)

print(ver)

token = os.environ["GITHUB_TOKEN"]

tarball_url = f"https://github.com/robotraconteur/robotraconteur/archive/{ref}.tar.gz"

print("Begin download")

req = urllib.request.Request(tarball_url, headers={
                             "Accept": "application/octet-stream", "Authorization": f"token {token}"})
response = urllib.request.urlopen(req)
file_data = response.read()

print("End download")

tarball_sha512 = hashlib.sha512(file_data).hexdigest()

print(tarball_sha512)

template_dict = {"VERSION": ver, "VERSION_REF": ref,
                 "TARBALL_SHA512": tarball_sha512}

with open('robotraconteur/packaging/vcpkg/CONTROL.in') as f:
    control_input = f.read()

with open('robotraconteur/packaging/vcpkg/portfile.cmake.in') as f:
    portfile_input = f.read()

control_file = control_input.replace("@{VERSION}", ver)
print(control_file)
portfile_file = portfile_input.replace(
    "@{VERSION_REF}", ref).replace("@{TARBALL_SHA512}", tarball_sha512)
print(portfile_file)

Path('vcpkg-robotraconteur/ports/robotraconteur').mkdir(exist_ok=True, parents=True)

with open('vcpkg-robotraconteur/ports/robotraconteur/CONTROL', 'w') as f:
    f.write(control_file)

with open('vcpkg-robotraconteur/ports/robotraconteur/portfile.cmake', 'w') as f:
    f.write(portfile_file)

shutil.copy('robotraconteur/packaging/vcpkg/static-build.patch',
            'vcpkg-robotraconteur/ports/robotraconteur/static-build.patch')



# robotraconteur_crosscheck is a tool to check cross-platform compatibility over the network

import RobotRaconteur as RR
import argparse
import json
import socket
import sys
from contextlib import suppress

_crosscheck_robdef = """
service experimental.crosscheck

object Crosscheck
    property string version_robotraconteur [readonly]
    property string hostname [readonly]
    property string platform [readonly]
end
"""


class crosscheck_impl:
    def __init__(self, node):
        self.node = node

    @property
    def version_robotraconteur(self):
        return self.node.RobotRaconteurVersion

    @property
    def hostname(self):
        return socket.gethostname()

    @property
    def platform(self):
        return sys.platform


class crosscheck_server_nodes:
    def __init__(self, args):
        self.insecure_node = None
        self.secure_node = None
        self.insecure_node_setup = None
        self.secure_node_setup = None
        self.insecure_obj = None
        self.secure_obj = None

        # self.secure_obj = crosscheck_impl()

        self.insecure_node = RR.RobotRaconteurNode()
        self.insecure_node.Init()
        self.insecure_obj = crosscheck_impl(self.insecure_node)
        self.insecure_node.RegisterServiceType(_crosscheck_robdef)
        self.insecure_node.RegisterService(
            "crosscheck", "experimental.crosscheck.Crosscheck", self.insecure_obj)
        self.insecure_node_setup = RR.ServerNodeSetup(args.insecure_nodename, args.insecure_tcp_port,
                                                      node=self.insecure_node)

        if args.secure_enabled:
            self.secure_node = RR.RobotRaconteurNode()
            self.secure_node.Init()
            self.secure_obj = crosscheck_impl(self.secure_node)
            self.secure_node.RegisterServiceType(_crosscheck_robdef)
            self.secure_node.RegisterService(
                "crosscheck", "experimental.crosscheck.Crosscheck", self.secure_obj)
            self.secure_node_setup = RR.SecureServerNodeSetup(args.secure_nodename, args.secure_tcp_port,
                                                              node=self.secure_node)

    def close(self):
        if self.insecure_node_setup:
            self.insecure_node_setup.close()

        if self.secure_node_setup:
            self.secure_node_setup.close()


class crosscheck_client:
    def __init__(self, json=False):

        client_node_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_TCP_TRANSPORT \
            | RR.RobotRaconteurNodeSetupFlags_TCP_TRANSPORT_IPV6_DISCOVERY \
            | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
            | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE

        self.json = json
        self.node = RR.RobotRaconteurNode()
        self.node.Init()
        self.node_setup = RR.RobotRaconteurNodeSetup(
            "", 0, client_node_flags, node=self.node)

        insecure_sub_filter = RR.ServiceSubscriptionFilter()
        insecure_sub_filter.TransportSchemes = ["rr+tcp"]
        self.insecure_sub = self.node.SubscribeServiceByType(
            "experimental.crosscheck.Crosscheck", insecure_sub_filter)

        secure_sub_filter = RR.ServiceSubscriptionFilter()
        secure_sub_filter.TransportSchemes = ["rrs+tcp"]
        self.secure_sub = self.node.SubscribeServiceByType(
            "experimental.crosscheck.Crosscheck", secure_sub_filter)

        self.timer = self.node.CreateTimer(1, self.timer_cb)
        self.timer.Start()

    def timer_cb(self, timer_event):
        if not self.json:
            insecure_clients = self.insecure_sub.GetConnectedClients()
            print(f"Insecure connected client count: {len(insecure_clients)}")
            for c in insecure_clients.values():
                print(f"    Node Name: {self.node.GetServiceNodeName(c)}")
                print(f"    Node ID: {self.node.GetServiceNodeID(c)}")
                print(f"    RR Version: {c.version_robotraconteur}")
                print(f"    Hostname: {c.hostname}")
                print(f"    Platform: {c.platform}")
                print()
            secure_clients = self.secure_sub.GetConnectedClients()
            print(f"Secure connected client count: {len(secure_clients)}")
            for c in secure_clients.values():
                print(f"    Node Name: {self.node.GetServiceNodeName(c)}")
                print(f"    Node ID: {self.node.GetServiceNodeID(c)}")
                print(f"    RR Version: {c.version_robotraconteur}")
                print(f"    Hostname: {c.hostname}")
                print(f"    Platform: {c.platform}")
                print()
            print()
        else:
            insecure_clients = self.insecure_sub.GetConnectedClients()
            insecure_clients_list = []
            for c in insecure_clients.values():
                insecure_clients_list.append({
                    "nodename": self.node.GetServiceNodeName(c),
                    "nodeid": str(self.node.GetServiceNodeID(c)),
                    "version_robotraconteur": c.version_robotraconteur,
                    "hostname": c.hostname,
                    "platform": c.platform
                })
            secure_clients = self.secure_sub.GetConnectedClients()
            secure_clients_list = []
            for c in secure_clients.values():
                secure_clients_list.append({
                    "nodename": self.node.GetServiceNodeName(c),
                    "nodeid": str(self.node.GetServiceNodeID(c)),
                    "version_robotraconteur": c.version_robotraconteur,
                    "hostname": c.hostname,
                    "platform": c.platform
                })
            print(json.dumps({
                "insecure": insecure_clients_list,
                "secure": secure_clients_list
            }))
            sys.stdout.flush()

    def close(self):
        if self.node_setup:
            self.node_setup.close()


def main():

    parser = argparse.ArgumentParser(
        description="tool to check Robot Raconteur cross-platform compatibility over the network")
    parser.add_argument("--servers-enabled", type=bool,
                        default=True, help="Enable server nodes")
    parser.add_argument("--insecure-nodename", type=str,
                        default="crosscheck", help="Node name for insecure server node")
    parser.add_argument("--insecure-tcp-port", type=int, default=0,
                        help="TCP listen port for insecure server node")
    parser.add_argument("--secure-enabled", type=bool,
                        default=True, help="Enable secure server node")
    parser.add_argument("--secure-nodename", type=str,
                        default="crosscheck_secure", help="Node name for secure server node")
    parser.add_argument("--secure-tcp-port", type=int, default=0,
                        help="TCP lister port for secure server node")
    parser.add_argument("--json", action="store_true",
                        help="Output in JSON format")

    args = parser.parse_args()

    servers = None
    client = None

    try:
        servers = crosscheck_server_nodes(args)
        client = crosscheck_client(json=args.json)
        print("Press enter to quit")
        input()

    finally:
        if servers:
            servers.close()
        if client:
            client.close()


if __name__ == "__main__":
    main()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'Getting Started with Robot Raconteur'
copyright = '2023, Wason Technology, LLC'
author = 'John Wason'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx_rtd_theme',
    'sphinx_toolbox.collapse',
    'sphinx_tabs.tabs'
]

templates_path = ['templates']
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']



# subscribe_type.py - Subscribe to services by type

from RobotRaconteur.Client import *
import time

# Subscribe to a service by type
root_object_type = "experimental.reynard_the_robot.Reynard"
sub = RRN.SubscribeServiceByType(root_object_type)
# Give a few seconds for the service to be connected
time.sleep(6)

# Run a function on all connected services. Could also use GetDefaultClientWait() to
# get the first connected client and run functions on that client
# without waiting for all clients to connect
connected_clients = sub.GetConnectedClients()
for k, c in connected_clients.items():
    print(f"Saying hello on client {k}")
    c.say("Hello from type subscription!")



import subprocess
from pathlib import Path
import re
import shutil
import platform

ios_triplets = [
    "arm-ios",
    "arm64-ios",
    "x64-ios"
]


archs = {
    "arm-ios": "armv7",
    "arm64-ios": "arm64",
    "x86-ios": "x86",
    "x64-ios": "x86_64",
    "arm64-iphonesimulator": "arm64"
}

src_root = Path(__file__).parent.parent.parent.absolute()

# Make sure we are in the right place!
rr_header = src_root / "RobotRaconteurCore/include/RobotRaconteur/RobotRaconteurConfig.h"
with open(rr_header) as f:
    f1 = f.read()
ver_str_m = re.search(
    "ROBOTRACONTEUR_VERSION_TEXT \"(\\d+\\.\\d+\\.\\d+)\"", f1)
ver_str = ver_str_m.group(1)
print("version " + ver_str)

build_dir = src_root / "build_xcframework"
build_dir.mkdir(exist_ok=True)

vcpkg_dir = build_dir / "vcpkg"

if not vcpkg_dir.is_dir():
    subprocess.check_call(["git", "clone", "--depth=1",
                          "https://github.com/microsoft/vcpkg.git"], cwd=build_dir)

if not (vcpkg_dir / "vcpkg").is_file():
    subprocess.check_call("./bootstrap-vcpkg.sh", cwd=vcpkg_dir)

# shutil.copy(src_root / "packaging/ios/CMakeLists-openssl-unix.txt",
#     build_dir / "vcpkg/ports/openssl/unix/CMakeLists.txt")

triplets_dir = src_root / "packaging/ios/triplets"
ports_in_dir = src_root / "packaging/ios/ports"
ports_dir = build_dir / "ports"
ports_rr_dir = ports_dir / "robotraconteur"
ports_rr_dir.mkdir(exist_ok=True, parents=True)

with open(ports_in_dir / "robotraconteur/vcpkg.json.in") as f:
    vcpkg_json_input = f.read()

vcpkg_json_file = vcpkg_json_input.replace("@{VERSION}", ver_str)
print(vcpkg_json_file)
with open(ports_rr_dir / "vcpkg.json", "w") as f:
    f.write(vcpkg_json_file)
shutil.copy(ports_in_dir / "robotraconteur/portfile.cmake",
            ports_rr_dir / "portfile.cmake")

combined_lib_dir = build_dir / "combined_lib"
combined_lib_dir_tmp = combined_lib_dir / "tmp"
combined_lib_dir.mkdir(exist_ok=True, parents=True)
combined_lib_dir_tmp.mkdir(exist_ok=True, parents=True)

for triplet in ios_triplets:
    subprocess.check_call(["./vcpkg", "install", f"--triplet={triplet}",
                           f"--overlay-ports={ports_dir}", f"--overlay-triplets={triplets_dir}", "robotraconteur"], cwd=vcpkg_dir)

    combined_lib_arch = archs[triplet]
    combined_lib_path = (combined_lib_dir_tmp /
                         f"robotraconteur_and_deps-{triplet}.a").absolute()
    subprocess.check_call([f"libtool -static -o {combined_lib_path} *.a"],
                          shell=True, cwd=vcpkg_dir / f"installed/{triplet}/debug/lib")

combined_lipo_ios = ("arm64-ios", "arm-ios")


combined_lipo_sim = ("x64-ios",)


lib_args = " ".join(
    [f"robotraconteur_and_deps-{c1}.a" for c1 in combined_lipo_ios])
subprocess.check_call(
    f"lipo {lib_args} -create -output ../robotraconteur_and_deps-ios.a", shell=True, cwd=combined_lib_dir_tmp)
sim_lib_args = " ".join(
    [f"robotraconteur_and_deps-{c1}.a" for c1 in combined_lipo_sim])
subprocess.check_call(
    f"lipo {sim_lib_args} -create -output ../robotraconteur_and_deps-iphonesimulator.a", shell=True, cwd=combined_lib_dir_tmp)

(build_dir / "robotraconteur.xcframework/ios").mkdir(exist_ok=True, parents=True)
(build_dir / "robotraconteur.xcframework/ios-simulator").mkdir(exist_ok=True, parents=True)

with open(src_root / "packaging/ios/Info.plist", "r") as f:
    plist_in = f.read()

if 'x64-ios' in triplet:
    plist = plist_in.replace(
        "<!--<string>x86_64</string>-->", "<string>x86_64</string>")
else:
    plist = plist_in.replace(
        "<!--<string>arm64</string>-->", "<string>arm64</string>")

with open(build_dir / "robotraconteur.xcframework/Info.plist", "w") as f:
    f.write(plist)

# shutil.copy(src_root / "packaging/ios/Info.plist", build_dir / "robotraconteur.xcframework/Info.plist")



##
# Command the ABB IRB 1200 robot using the position_command wire
#

from RobotRaconteur.Client import *
import time
import numpy as np
import sys

# URL for connecting to the robot. By default connects to UR5e simulated robot.
url = 'rr+tcp://localhost:52511?service=robot'
if (len(sys.argv) >= 2):
    url = sys.argv[1]

# Connect to the robot driver
c = RRN.ConnectService(url)

# Retrieve the current robot state and print the current command mode
print(c.robot_state.PeekInValue()[0].command_mode)

# Retrieve the constants for the com.robotraconteur.robotics.robot service definition
robot_const = RRN.GetConstants("com.robotraconteur.robotics.robot", c)

# Retrieve the "halt" and "position_command" enum values
halt_mode = robot_const["RobotCommandMode"]["halt"]
position_mode = robot_const["RobotCommandMode"]["position_command"]

# Retreive the structure type to create RobotJointCommand objects
RobotJointCommand = RRN.GetStructureType("com.robotraconteur.robotics.robot.RobotJointCommand", c)

# Change the robot command mode, first to halt, then to position_mode
c.command_mode = halt_mode
time.sleep(0.1)
c.command_mode = position_mode
time.sleep(0.1)

# Connect to the position_command and robot_state wires for real-time data streaming
cmd_w = c.position_command.Connect()
state_w = c.robot_state.Connect()
# Set a lifespan of 0.5 seconds for the robot state. If new packets
# are not received within 0.5 seconds, an exception will be thrown
state_w.InValueLifespan = 0.5

# Wait for the state_w wire to receive valid data
state_w.WaitInValueValid()

# Initialize command_seqno. command_seqno must increment for each position command
command_seqno = 1

# Retreive the current joint position
init_joint_pos = state_w.InValue.joint_position

t0 = time.time()

while (True):
    t = time.time() - t0

    # Retreive the current robot state
    robot_state = state_w.InValue

    # Make sure the robot is still in position mode
    if (robot_state.command_mode != position_mode):
        raise Exception("Robot is not in position mode")

    # Increment command_seqno
    command_seqno += 1

    # Create Fill the RobotJointCommand structure
    joint_cmd1 = RobotJointCommand()
    joint_cmd1.seqno = command_seqno  # Strictly increasing command_seqno
    joint_cmd1.state_seqno = robot_state.seqno  # Send current robot_state.seqno as failsafe

    # Generate a joint command, in this case a sin wave
    cmd = 0.2 * np.array([1, 0, 0, 0, 0, 0]) * np.sin(t) + init_joint_pos
    # Set the joint command
    joint_cmd1.command = cmd

    # Send the joint command to the robot
    cmd_w.OutValue = joint_cmd1

    # Wait for next iteration
    time.sleep(.01)



import RobotRaconteur as RR
import pytest
import time
import threading

_sub_test_service_def = """
service experimental.sub_test

object testobj
    function double add_two_numbers(double a, double b)
end
"""

intra_server_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_INTRA_TRANSPORT_START_SERVER \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_ANNOUNCE \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE

intra_client_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE


class _testobj_impl():

    def add_two_numbers(self, a, b):
        return a + b


class _testservice_impl():

    def __init__(self, nodename, nodeid):
        self._obj = _testobj_impl()
        self._node = RR.RobotRaconteurNode()
        self._node.SetNodeID(nodeid)
        self._node.Init()

        self._node.RegisterServiceType(_sub_test_service_def)
        self._node.RegisterService(
            "test_service", "experimental.sub_test.testobj", self._obj)

        self._node_setup = RR.RobotRaconteurNodeSetup(
            nodename, 0, flags=intra_server_flags, node=self._node)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._node_setup.ReleaseNode()
        self._node.Shutdown()
        self._node_setup = None
        self._node = None


def test_subscribe_by_type():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    server1 = _testservice_impl("server1", test_servers["server1"])

    with client_node_setup, server1:
        sub = client_node.SubscribeServiceByType(
            "experimental.sub_test.testobj")
        c = sub.GetDefaultClientWait(5)
        assert c.add_two_numbers(1, 2) == 3
        c2 = sub.GetDefaultClient()
        assert c2.add_two_numbers(3, 2) == 5
        res, c3 = sub.TryGetDefaultClient()
        assert res
        assert c3.add_two_numbers(3, 2) == 5
        res, c4 = sub.TryGetDefaultClientWait(1)
        assert res
        assert c4.add_two_numbers(3, 7) == 10

        c5 = sub.GetConnectedClients()
        assert len(c5) >= 1
        c6 = c5[RR.ServiceSubscriptionClientID(
            test_servers["server1"], "test_service")]
        assert c6.add_two_numbers(3, 7) == 10

        async_client_called = [False]
        async_client_called_evt = threading.Event()

        def async_default_client_handler(client, err):
            assert err is None
            assert client.add_two_numbers(3, 9) == 12
            async_client_called[0] = True
            async_client_called_evt.set()

        sub.AsyncGetDefaultClient(async_default_client_handler, 5)
        async_client_called_evt.wait(5)
        assert async_client_called[0]

        connect_called = [False]
        disconnect_called = [False]

        connect_called_evt = threading.Event()
        disconnect_called_evt = threading.Event()

        def connect_handler(sub, client_id, client):
            assert client_id.NodeID == test_servers["server2"]
            assert client_id.ServiceName == "test_service"
            connect_called[0] = True
            connect_called_evt.set()

        def disconnect_handler(sub, client_id, client):
            assert client_id.NodeID == test_servers["server2"]
            assert client_id.ServiceName == "test_service"
            disconnect_called[0] = True
            disconnect_called_evt.set()

        sub.ClientConnected += connect_handler
        sub.ClientDisconnected += disconnect_handler

        server2 = _testservice_impl("server2", test_servers["server2"])
        with server2:
            connect_called_evt.wait(5)
            assert connect_called[0]

        disconnect_called_evt.wait(5)
        assert disconnect_called[0]

        sub.ClientConnected -= connect_handler
        sub.ClientDisconnected -= disconnect_handler


def test_subscribe_serviceinfo2():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    server1 = _testservice_impl("server1", test_servers["server1"])

    with client_node_setup, server1:
        sub = client_node.SubscribeServiceInfo2(
            "experimental.sub_test.testobj")

        wait_count = 0
        while True:
            time.sleep(0.1)
            if len(sub.GetDetectedServiceInfo2()) > 0:
                break
            wait_count += 1
            if wait_count > 50:
                raise Exception("Timeout waiting for service info")

        detected_nodes = sub.GetDetectedServiceInfo2()
        assert len(detected_nodes) >= 1
        service_info = detected_nodes[RR.ServiceSubscriptionClientID(
            test_servers["server1"], "test_service")]
        assert service_info.NodeID == test_servers["server1"]
        assert service_info.Name == "test_service"
        assert service_info.RootObjectType == "experimental.sub_test.testobj"
        assert service_info.Name == "test_service"

        connect_called = [False]
        # disconnect_called = [False]
        connect_called_evt = threading.Event()
        # disconnect_called_evt = threading.Event()

        def connect_handler(sub, client_id, service_info2):
            assert client_id.NodeID == test_servers["server2"]
            assert client_id.ServiceName == "test_service"
            connect_called[0] = True
            connect_called_evt.set()

        # disconnect called takes up to 1 minute

        # def disconnect_handler(sub, client_id, client):
        #     assert client_id.NodeID == test_servers["server2"]
        #     assert client_id.ServiceName == "test_service"
        #     disconnect_called[0] = True
        #     disconnect_called_evt.set()

        sub.ServiceDetected += connect_handler
        # sub.ClientDisconnected += disconnect_handler

        server2 = _testservice_impl("server2", test_servers["server2"])
        with server2:
            connect_called_evt.wait(5)
            assert connect_called[0]

        # disconnect_called_evt.wait(5)
        # assert disconnect_called[0]

        sub.ServiceDetected -= connect_handler
        # sub.ClientDisconnected -= disconnect_handler


def test_subscribe_by_url():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    with client_node_setup:

        sub = client_node.SubscribeService(
            "rr+intra:///?nodename=server2&service=test_service")

        connect_called = [False]
        disconnect_called = [False]

        connect_called_evt = threading.Event()
        disconnect_called_evt = threading.Event()

        def connect_handler(sub, client_id, client):
            assert client_id.NodeID == test_servers["server2"]
            assert client_id.ServiceName == "test_service"
            connect_called[0] = True
            connect_called_evt.set()

        def disconnect_handler(sub, client_id, client):
            assert client_id.NodeID == test_servers["server2"]
            assert client_id.ServiceName == "test_service"
            disconnect_called[0] = True
            disconnect_called_evt.set()

        sub.ClientConnected += connect_handler
        sub.ClientDisconnected += disconnect_handler

        server2 = _testservice_impl("server2", test_servers["server2"])
        with server2:
            connect_called_evt.wait(5)
            assert connect_called[0]

            c = sub.GetDefaultClient()
            assert c.add_two_numbers(3, 2) == 5

        disconnect_called_evt.wait(5)
        assert disconnect_called[0]

        sub.ClientConnected -= connect_handler
        sub.ClientDisconnected -= disconnect_handler


def test_subscribe_by_url_bad_url():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    with client_node_setup:

        connect_err_called = [False]
        connect_err_called_evt = threading.Event()

        def connect_err_handler(sub, id, url, err):
            assert err is not None
            assert "RobotRaconteur.ConnectionError" in str(err)
            assert "rr+intra:///?nodename=server5&service=test_service" in url
            connect_err_called[0] = True
            connect_err_called_evt.set()

        # Pass an invalid URL and make sure error is called
        sub = client_node.SubscribeService(
            "rr+intra:///?nodename=server5&service=test_service")

        sub.ClientConnectFailed += connect_err_handler

        connect_err_called_evt.wait(15)
        assert connect_err_called[0]

        sub.ClientConnectFailed -= connect_err_handler
        time.sleep(0.1)



# reynard_quickstart_service.py

import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s

import threading
import reynard_the_robot

# Define the service definition for the quickstart service
reynard_quickstart_interface = """
service experimental.reynard_quickstart

object ReynardQuickstart
    function void say(string text)
    function void teleport(double x, double y)
end
"""

# Implement the quickstart service


class ReynardQuickstartImpl(object):
    def __init__(self):
        self.reynard = reynard_the_robot.Reynard()
        self.reynard.start()
        self._lock = threading.Lock()

    def say(self, text):
        with self._lock:
            self.reynard.say(text)

    def teleport(self, x, y):
        with self._lock:
            self.reynard.teleport(x, y)


with RR.ServerNodeSetup("experimental.reynard_quickstart", 53222):
    # Register the service type
    RRN.RegisterServiceType(reynard_quickstart_interface)

    reynard_inst = ReynardQuickstartImpl()

    # Register the service
    RRN.RegisterService("reynard", "experimental.reynard_quickstart.ReynardQuickstart", reynard_inst)

    # Wait for program exit to quit
    input("Press enter to quit")



import pytest
from .ServiceTestClient import ServiceTestClient
import os
from RobotRaconteur.Client import *
import sys


def _member_test(fn, test_server_node_config):
    service_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService")
    c = ServiceTestClient()
    c.ConnectService(service_url)
    try:
        fn(c)
    finally:
        c.DisconnectService()


def test_properties(test_server_node_config):
    _member_test(lambda c: c.TestProperties(), test_server_node_config)


def test_functions(test_server_node_config):
    _member_test(lambda c: c.TestFunctions(), test_server_node_config)


def test_events(test_server_node_config):
    _member_test(lambda c: c.TestEvents(), test_server_node_config)


def test_objrefs(test_server_node_config):
    _member_test(lambda c: c.TestObjRefs(), test_server_node_config)


def test_pipes(test_server_node_config):
    _member_test(lambda c: c.TestPipes(), test_server_node_config)


def test_callbacks(test_server_node_config):
    _member_test(lambda c: c.TestCallbacks(), test_server_node_config)


def test_wires(test_server_node_config):
    _member_test(lambda c: c.TestWires(), test_server_node_config)


def test_memories(test_server_node_config):
    if sys.version_info < (3, 0):
        return
    _member_test(lambda c: c.TestMemories(), test_server_node_config)


def test_authentication(test_server_node_config):
    service_auth_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService_auth")
    c = ServiceTestClient()
    c.TestAuthentication(service_auth_url)


def test_object_lock(test_server_node_config):
    service_auth_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService_auth")
    c = ServiceTestClient()
    c.TestObjectLock(service_auth_url)


def test_monitor_lock(test_server_node_config):
    service_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService")
    c = ServiceTestClient()
    c.TestMonitorLock(service_url)


def test_async(test_server_node_config):
    service_auth_url = test_server_node_config.get_service_url(
        "RobotRaconteurTestService_auth")
    c = ServiceTestClient()
    c.TestAsync(service_auth_url)



import shutil
import subprocess
from pathlib import Path
import glob
import tempfile

asset_dir = "artifacts/main"

with open(asset_dir + '/out-win-x86/rrversion.txt') as f:
    ver_str = f.read().strip()

with open('robotraconteur/packaging/nuget/nuspec.in') as f:
    nuspec_input = f.read()

nuspec_file = nuspec_input.replace("@{VERSION}", ver_str)
print(nuspec_file)

with tempfile.TemporaryDirectory() as tmpdirname:

    tmpdir = Path(tmpdirname)

    with open(tmpdir.joinpath('.nuspec'), 'w') as f:
        f.write(nuspec_file)

    win32dir = f'{asset_dir}/out-win-x86'
    win64dir = f'{asset_dir}/out-win-x64'

    tmpdir.joinpath('build/net45').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath('build/netstandard2.0').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath('lib/net45').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath('lib/netstandard2.0').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath(
        'runtimes/win-x64/native').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath(
        'runtimes/win-x86/native').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath('tools').mkdir(exist_ok=True, parents=True)
    tmpdir.joinpath('docs').mkdir(exist_ok=True, parents=True)

    shutil.copyfile('robotraconteur/packaging/nuget/RobotRaconteurNET-net45.targets.in',
                    tmpdir.joinpath('build/net45/RobotRaconteurNET.targets'))
    shutil.copyfile('robotraconteur/packaging/nuget/RobotRaconteurNET-netstandard2.0.targets.in',
                    tmpdir.joinpath('build/netstandard2.0/RobotRaconteurNET.targets'))

    shutil.copyfile(f'{win32dir}/NET/RobotRaconteurNET.dll',
                    tmpdir.joinpath('lib/net45/RobotRaconteurNET.dll').absolute())
    shutil.copyfile(f'{win64dir}/NET/netstandard/RobotRaconteurNET.dll',
                    tmpdir.joinpath('lib/netstandard2.0/RobotRaconteurNET.dll').absolute())
    shutil.copyfile(f'{win32dir}/NET/RobotRaconteurNET.xml',
                    tmpdir.joinpath('lib/net45/RobotRaconteurNET.xml').absolute())
    shutil.copyfile(f'{win64dir}/NET/netstandard/RobotRaconteurNET.xml',
                    tmpdir.joinpath('lib/netstandard2.0/RobotRaconteurNET.xml').absolute())
    shutil.copyfile(f'{win32dir}/NET/Native/RobotRaconteurNETNative.dll',
                    tmpdir.joinpath('runtimes/win-x86/native/RobotRaconteurNETNative.dll').absolute())
    shutil.copyfile(f'{win64dir}/NET/Native/RobotRaconteurNETNative.dll',
                    tmpdir.joinpath('runtimes/win-x64/native/RobotRaconteurNETNative.dll').absolute())
    shutil.copyfile(f'{win32dir}/bin/RobotRaconteurGen.exe',
                    tmpdir.joinpath('tools/RobotRaconteurGen.exe').absolute())
    shutil.copyfile('robotraconteur/RobotRaconteurNET/README.md',
                    tmpdir.joinpath('docs/README.md').absolute())

    subprocess.check_call('nuget pack', cwd=tmpdir, shell=True)

    for file in glob.glob(str(tmpdir.joinpath('*.nupkg').absolute())):
        print(file)
        shutil.move(file, 'distfiles/')



# authentication_example.py - Example of using authentication in Python

from RobotRaconteur.Client import *

url = 'rr+tcp://localhost:53226?service=authentication_example'

# Create the user credentials
username = "testuser1"
cred = {"password": RR.VarValue("testpass1", "string")}

# Connect to the service
c = RRN.ConnectService(url, username, cred)

# Use the service
c.say("Message to secure service")



from RobotRaconteur.Client import *
import sys
import threading
import functools
import traceback

try:
    xrange  # type: ignore
except NameError:
    xrange = range

try:
    raw_input  # type: ignore
except NameError:
    raw_input = input


def main():
    url = sys.argv[1]

    servicetest_count = [0]
    servicetest_connectcount = [0]
    servicetest_keepgoing = True

    servicetest_lock = threading.Lock()

    def servicetest1(o, exp):
        if (exp is not None):
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(exp))
        try:
            o.async_func3(1, 2, functools.partial(servicetest2, o))
        except Exception as ee:
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(ee))

    def servicetest2(o, d, exp):
        if (exp is not None):
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(exp))
        try:
            o.async_func3(1, 2, functools.partial(servicetest2, o))
        except Exception as ee:
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(ee))

    def servicetest3(url1, obj, exp):
        if (exp is not None):
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(exp))
        try:
            RRN.AsyncDisconnectService(
                obj, functools.partial(servicetest4, url))
        except Exception as ee:
            if (servicetest_keepgoing):
                sys.exit("Got exception: " + str(ee))

    def servicetest4(url):

        RRN.AsyncConnectService(url, None, None, None,
                                functools.partial(servicetest3, url))

    def servicetest5(p, w, ev):
        try:
            if (ev.stopped):
                return
            p.AsyncSendPacket(servicetest_count[0], lambda pnum, err: None)

            for i in xrange(100):
                d = servicetest_count[0] * 100 + i
                w.OutValue = d

            servicetest_count[0] += 1
        except:
            traceback.print_exc()

    def servicetest7(e):
        with servicetest_lock:
            while p.Available > 0:
                p.ReceivePacket()

    o = RRN.ConnectService(url)
    o.async_func3(1, 2, functools.partial(servicetest2, o))

    p = o.broadcastpipe.Connect(-1)
    p.PacketReceivedEvent += servicetest7
    w = o.broadcastwire.Connect()

    # tt=RobotRaconteurNode.s.CreateTimer(.04,functools.partial(servicetest5,p,w))
    tt = RR.WallTimer(.04, functools.partial(servicetest5, p, w))
    tt.Start()

    print("Press enter to stop")
    a = raw_input()

    servicetest_keepgoing = False

    tt.Stop()
    RR.DisconnectService(o)


if __name__ == "__main__":
    main()




import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s
import time


def ev1():
    print("Event 1")


def ev2(a, b):
    print("Event2 : " + str(a) + " " + str(b))


def main():
    RRN.RegisterTransport(RR.LocalTransport())
    RRN.RegisterTransport(RR.TcpTransport())

    cred1 = {"password": RR.RobotRaconteurVarValue("testpass1", "string")}
    c = RRN.ConnectService(
        'tcp://localhost:4343/{0}/MatlabTestService', "testuser1", cred1)

    c.testevent1 += ev1
    c.testevent2 += ev2

    c.testfunc1(10, 20)
    print(c.testprop1)
    c.testprop1 = 30
    print(c.testfunc2(20))

    print(c.testprop2)
    s = RRN.NewStructure('example.MatlabTestService.MatlabTestStruct', c)
    s.val1 = 38274
    s.val2 = [1.234, 3.14]

    c.testprop2 = s
    c2 = c.testprop2
    print(c2.val1)
    print(c2.val2)

    time.sleep(2)

    RRN.DisconnectService(c)


if __name__ == '__main__':
    main()



import sys
import shutil
import subprocess
import re
from pathlib import Path
import os
import glob
import argparse


# cSpell: disable

def main():

    parser = argparse.ArgumentParser(description="Build MATLAB locally")
    parser.add_argument("--matlab-version", type=str,
                        default="R2020a", help="MATLAB version to use")
    parser.add_argument("--matlab-root", type=str, default=None,
                        help="MATLAB root location (default is searched)")
    parser.add_argument("--semver-full", type=str,
                        default=None, help="Full semver of release")
    parser.add_argument("--no-vcpkg", action='store_true')
    parser.add_argument("--no-clean", action='store_true')

    args = parser.parse_args()

    config_header = Path(
        "RobotRaconteurCore/include/RobotRaconteur/RobotRaconteurConfig.h").absolute()
    assert config_header.is_file()

    if args.semver_full is not None:
        ver_str = args.semver_full
        semver_tag_regex = r"^(((?:0|[1-9]\d*)\.(?:0|[1-9]\d*)\.(?:0|[1-9]\d*))(-(?:alpha|beta|rc)\d+)?)"
        m = re.match(semver_tag_regex, ver_str)
        assert m, f"Invalid semver-full {ver_str}"
    else:
        with open(config_header) as f:
            f1 = f.read()
        ver_str_m = re.search(
            "ROBOTRACONTEUR_VERSION_TEXT \"(\\d+\\.\\d+\\.\\d+)\"", f1)
        ver_str = ver_str_m.group(1)
        print("version " + ver_str)

    build_path = Path("build").absolute()

    if not args.no_clean:
        if build_path.is_dir():
            shutil.rmtree(str(build_path))

    if sys.platform == "win32":
        vcpkg_triplet = "x64-windows-static-md"
    elif sys.platform == "darwin":
        vcpkg_triplet = "x64-osx"
    else:
        vcpkg_triplet = "x64-linux"

    vcpkg_path = Path("vcpkg").absolute()
    if not args.no_vcpkg:

        vcpkg_libs = "boost-algorithm " \
                     "boost-array boost-asio "\
                     "boost-assign boost-atomic boost-bind boost-config boost-container boost-date-time " \
                     "boost-smart-ptr boost-filesystem boost-foreach boost-format boost-function boost-interprocess " \
                     "boost-intrusive boost-lexical-cast boost-locale boost-random boost-range boost-regex " \
                     "boost-scope-exit boost-signals2 boost-thread boost-tuple boost-unordered " \
                     "boost-utility boost-uuid boost-program-options"

        if vcpkg_path.is_dir():
            assert vcpkg_path.joinpath(".git").is_dir()
            subprocess.check_call("git pull", shell=True, cwd=vcpkg_path)
        else:
            subprocess.check_call(
                "git clone --depth=1 https://github.com/microsoft/vcpkg.git", shell=True)
        if sys.platform == "win32":
            subprocess.check_call("bootstrap-vcpkg.bat",
                                  shell=True, cwd=vcpkg_path)
            subprocess.check_call(
                f"vcpkg install --triplet {vcpkg_triplet} {vcpkg_libs}", shell=True, cwd=vcpkg_path)
        else:
            subprocess.check_call("./bootstrap-vcpkg.sh",
                                  shell=True, cwd=vcpkg_path)
            subprocess.check_call(
                f"./vcpkg install --triplet {vcpkg_triplet} {vcpkg_libs}", shell=True, cwd=vcpkg_path)

    matlab_root = None

    if args.matlab_root is not None:
        matlab_root = Path(args.matlab_root)
    else:
        matlab_ver = args.matlab_version

        assert re.match(r"^R\d{4}[a-z]$", matlab_ver)

        if sys.platform == "win32":
            import winreg
            mw_key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE, f"SOFTWARE\\MathWorks\\{matlab_ver}\\MATLAB")
            matlab_path1 = winreg.QueryValue(mw_key, None)
            mw_key.Close()
            matlab_path_m = re.match(r"^(.*)\\bin\\.*$", matlab_path1)
            assert matlab_path_m
            matlab_path = Path(matlab_path_m.group(1))
            assert matlab_path.joinpath("bin/matlab.exe").exists()
        elif sys.platform == "darwin":
            matlab_path = Path(f"/Applications/MATLAB_{matlab_ver}.app")
            assert matlab_path.joinpath("bin/matlab").exists()
        else:
            matlab_path = Path(os.environ("HOME")).joinpath(
                f"/MATLAB/{matlab_ver}")
            assert matlab_path.joinpath("bin/matlab").exists()

    vcpkg_toolchain_file = vcpkg_path.joinpath(
        "scripts/buildsystems/vcpkg.cmake").absolute()
    assert vcpkg_toolchain_file.exists()

    subprocess.check_call("cmake -G \"Ninja\" -DBUILD_GEN=ON -DBUILD_TESTING=ON -DBoost_USE_STATIC_LIBS=ON "
                          "-DCMAKE_BUILD_TYPE=Release -DBUILD_MATLAB_MEX=ON Boost_NO_SYSTEM_PATHS=ON "
                          f"-DVCPKG_TARGET_TRIPLET={vcpkg_triplet} "
                          f"-DCMAKE_TOOLCHAIN_FILE={vcpkg_toolchain_file} "
                          f"-DROBOTRACONTEUR_VERSION_SEMVER=\"{ver_str}\" "
                          "-S . -B build", shell=True)

    subprocess.check_call("cmake --build . --config Release",
                          shell=True, cwd=build_path)

    if sys.platform == "linux":
        machine = "glnxa64"
    elif sys.platform == "darwin":
        machine = "maci64"
    elif sys.platform == "win32":
        machine = "win64"
    else:
        machine = f"UNKNOWN-{sys.platform}"

    try:
        os.unlink(glob.glob(f"build/out/Matlab/*.lib")[0])
        os.unlink(glob.glob(f"build/out/Matlab/*.exp")[0])
        os.unlink(glob.glob(f"build/out/Matlab/*.pdb")[0])
    except:
        pass

    shutil.move(glob.glob("build/out/Matlab")
                [0], f"build/out/RobotRaconteur-{ver_str}-MATLAB-{machine}")
    shutil.copy("LICENSE.txt",
                f"build/out/RobotRaconteur-{ver_str}-MATLAB-{machine}")

    if sys.platform == "win32":
        subprocess.check_call(["zip", f"RobotRaconteur-{ver_str}-MATLAB-{machine}.zip",
                              "-FSr", f"RobotRaconteur-{ver_str}-MATLAB-{machine}"], cwd="build/out")
    else:
        subprocess.check_call(
            ["tar", "cvzf", f"RobotRaconteur-{ver_str}-MATLAB-{machine}.tar.gz", f"RobotRaconteur-{ver_str}-MATLAB-{machine}"], cwd="build/out")


if __name__ == "__main__":
    main()



from __future__ import print_function

from RobotRaconteur.Client import *


def test_nowutc():
    print(RRN.NowUTC())
    return


def test_logging():
    import datetime
    r = RR.RRLogRecord()
    r.Node = RRN
    r.Time = datetime.datetime.now()
    r.Level = RR.LogLevel_Warning
    r.Message = "This is a test warning"

    RRN.LogRecord(r)

    print(r)


def test_loghandler():
    log_rrn = RR.RobotRaconteurNode()
    log_rrn.Init()
    log_rrn.SetLogLevel(RR.LogLevel_Debug)
    user_log_handler = RR.UserLogRecordHandler(
        lambda x: print("python handler: " + str(x)))
    log_rrn.SetLogRecordHandler(user_log_handler)

    import datetime
    r = RR.RRLogRecord()
    r.Node = log_rrn
    r.Time = datetime.datetime.now()
    r.Level = RR.LogLevel_Warning
    r.Message = "This is a test warning"

    log_rrn.LogRecord(r)

    log_rrn.Shutdown()


def test_numpy_dtypes():

    np_node = RR.RobotRaconteurNode()
    np_node.Init()

    try:
        np_node.RegisterServiceTypeFromFile(
            "com.robotraconteur.testing.TestService2")
        np_node.RegisterServiceTypeFromFile(
            "com.robotraconteur.testing.TestService1")
        np_node.RegisterServiceTypeFromFile(
            "com.robotraconteur.testing.TestService3")

        np_node.GetNamedArrayDType(
            'com.robotraconteur.testing.TestService3.transform')
        np_node.GetConstants("com.robotraconteur.testing.TestService3")
        np_node.GetPodDType('com.robotraconteur.testing.TestService3.testpod2')
    finally:
        np_node.Shutdown()



# subscribe_url.py - Subscribe to a service using a URL

from RobotRaconteur.Client import *

url = 'rr+tcp://localhost:29200?service=reynard'
sub = RRN.SubscribeService(url)
c = sub.GetDefaultClientWait(10)
c.say("Hello from URL subscription!")



# pyri_device_manager_client.py - Example of using the PyriDeviceManagerClient class to connect to a device

# The package pyri-common must be installed to use this example. See https://github.com/pyri-project/pyri-core
# for more information.

from pyri.device_manager_client import DeviceManagerClient
from RobotRaconteur.Client import *
from RobotRaconteurCompanion.Util.ImageUtil import ImageUtil
import cv2

# Create the DeviceManagerClient object
d = DeviceManagerClient()

# Refresh the available devices from the service
d.refresh_devices(1)

# Get the default client and capture and image
c = d.get_device_client("camera", timeout=10)
raw_img = c.capture_frame()
image_util = ImageUtil(client_obj=c)

# Display the image
img = image_util.image_to_array(raw_img)
cv2.namedWindow("Image")
cv2.imshow("Image", img)
cv2.waitKey()
cv2.destroyAllWindows()



# Example iRobot Create client in Python

from RobotRaconteur.Client import *
import time
import numpy
import sys

# Function to call when "Bump" event occurs


def bumped():
    print("Bump!!")


def main():

    url = 'rr+tcp://localhost:22354?service=create'
    if (len(sys.argv) >= 2):
        url = sys.argv[1]

    # Connect to the service
    c = RRN.ConnectService(url)

    # Add a function handler for the "Bump" event
    c.bump += bumped

    # Demonstrate peeking a wire value
    state_val, _ = c.create_state.PeekInValue()
    print(state_val)

    # Connect a WireConnection to the "packets" wire
    wire = c.create_state.Connect()

    # Add a callback function for when the wire value changes
    wire.WireValueChanged += wire_changed

    # Set the play_callback function for this client
    c.claim_play_callback()
    c.play_callback.Function = play_callback

    # Drive a bit
    c.drive(0.1, 1)
    time.sleep(5)
    c.drive(0, 1)
    time.sleep(10)


# Function to call when the wire value changes
def wire_changed(w, value, time):

    val = w.InValue
    # Print the new value to the console.  Comment out this line
    # to see the other output more clearly
    print(val)

# Callback for when the play button is pressed on the Create


def play_callback(dist, angle):
    return numpy.array([69, 16, 60, 16, 69, 16], dtype='u1')


if __name__ == '__main__':
    main()



import pytest
from .ServiceTestClient import ServiceTestClient
import os
from RobotRaconteur.Client import *
import sys


def test_attributes(test_server_node_config):
    c = RRN.ConnectService(test_server_node_config.get_service_url("RobotRaconteurTestService"))

    attr = RRN.GetServiceAttributes(c)

    assert (len(attr) == 2)

    assert (attr["test"].data == "This is a test attribute")
    assert (attr["test2"].data[0] == 42)



# Python example Robot Raconteur service for Reynard the Robot using the HTTP REST API to communicate with the robot.

# Note that Reynard the Robot uses mm and degrees, while Robot Raconteur services are expected
# to use meters and radians. The service will convert the units as needed.

import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s
import numpy as np
from pathlib import Path
import requests
import drekar_launch_process

# Define the class that implements the service. In Python, the thunk code is dynamically handled
# so there is no need to generate the service definition.


class Reynard_impl:
    def __init__(self, base_url="http://localhost:29201"):
        self._base_url = base_url
        self._state_timer = None

        # Retrieve the structure type for ReynardState
        self._state_type = RRN.GetStructureType("experimental.reynard_the_robot.ReynardState")

        # Create the EventHook for new_message event
        self.new_message = RR.EventHook()

        # Wires are automatically created when the service is registered when [readonly] or [writeonly] is used

    def _start(self):
        self._state_timer = RRN.CreateTimer(0.1, self._timer_cb)
        self._state_timer.Start()

    def _stop(self):
        if self._state_timer is not None:
            self._state_timer.TryStop()
            self._state_timer = None

    @property
    def robot_position(self):
        res = requests.get(self._base_url + "/api/state")
        res.raise_for_status()
        res_json = res.json()
        return [res_json["x"] / 1000.0, res_json["y"] / 1000.0]

    @property
    def color(self):
        res = requests.get(self._base_url + "/api/color")
        res.raise_for_status()
        res_json = res.json()
        return [res_json["r"], res_json["g"], res_json["b"]]

    @color.setter
    def color(self, value):
        res = requests.post(self._base_url + "/api/color", json={"r": value[0], "g": value[1], "b": value[2]})
        res.raise_for_status()

    def teleport(self, x, y):
        res = requests.post(self._base_url + "/api/teleport", json={"x": x * 1000, "y": y * 1000})
        res.raise_for_status()

    def setf_arm_position(self, q1, q2, q3):
        res = requests.post(self._base_url + "/api/arm",
                            json={"q1": np.rad2deg(q1), "q2": np.rad2deg(q2), "q3": np.rad2deg(q3)})
        res.raise_for_status()

    def getf_arm_position(self):
        res = requests.get(self._base_url + "/api/arm")
        res.raise_for_status()
        res_json = res.json()
        return [np.deg2rad(res_json["q1"]), np.deg2rad(res_json["q2"]), np.deg2rad(res_json["q3"])]

    def drive_robot(self, vel_x, vel_y, timeout=-1, wait=False):
        res = requests.post(self._base_url + "/api/drive_robot",
                            json={"vel_x": vel_x * 1000.0, "vel_y": vel_y * 1000.0, "timeout": timeout, "wait": wait})
        res.raise_for_status()

    def drive_arm(self, q1, q2, q3, timeout=-1, wait=False):
        res = requests.post(self._base_url + "/api/drive_arm",
                            json={"q1": np.rad2deg(q1), "q2": np.rad2deg(q2), "q3": np.rad2deg(q3), "timeout": timeout, "wait": wait})
        res.raise_for_status()

    def say(self, message):
        res = requests.post(self._base_url + "/api/say", json={"message": message})
        res.raise_for_status()

    def _timer_cb(self, ev):
        try:
            res = requests.get(self._base_url + "/api/state")
            res.raise_for_status()
            res_json = res.json()
            state = self._state_type()
            state.time = res_json["time"]
            state.robot_position = 1e-3 * np.array([res_json["x"], res_json["y"]], dtype=np.float64)
            state.arm_position = np.deg2rad(
                np.array([res_json["q1"], res_json["q2"], res_json["q3"]], dtype=np.float64))
            state.robot_velocity = 1e-3 * np.array([res_json["vel_x"], res_json["vel_y"]], dtype=np.float64)
            state.arm_velocity = np.deg2rad(
                np.array([res_json["vel_q1"], res_json["vel_q2"], res_json["vel_q3"]], dtype=np.float64))

            if self.state is not None:
                self.state.OutValue = state

            message_res = requests.get(self._base_url + "/api/messages")
            message_res.raise_for_status()
            message_res_json = message_res.json()
            for message in message_res_json:
                self.new_message.fire(message)

        except Exception as e:
            RRN.LogMessage(RR.LogLevel_Warning, "Error in timer callback: " + str(e))


def main():

    # Set the base URL for the Reynard the Robot HTTP REST API
    base_url = "http://localhost:29201"

    # Use the robdef from a file. In practice, this is usually done using
    # a package resource. See RobotRaconteurCompanion.Util.RobDef.register_service_types_from_resources
    RRN.RegisterServiceTypesFromFiles(
        [str(Path(__file__).parent.parent.parent / "robdef" / "experimental.reynard_the_robot.robdef")])

    # Create the Reynard service instance
    reynard_obj = Reynard_impl(base_url)

    # Use RobotRaconteur.ServerNodeSetup to initialize Robot Raconteur using the default node
    with RR.ServerNodeSetup("experimental.reynard_the_robot_python_rest", 59201):

        # Register the service with Robot Raconteur
        ctx = RRN.RegisterService("reynard", "experimental.reynard_the_robot.Reynard", reynard_obj)

        # Start the service timer for updating the state
        reynard_obj._start()

        # Print some info for the user
        print("Reynard the Robot Python Service Started")
        print()
        print("Candidate connection urls:")
        ctx.PrintCandidateConnectionURLs()
        print()
        print("Press Ctrl-C to quit...")

        # Use drekar_launch_process package to wait for exit
        drekar_launch_process.wait_exit()

        # Stop the service timer
        reynard_obj._stop()


if __name__ == '__main__':
    main()



# Copyright 2011-2020 Wason Technology, LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import RobotRaconteur as RR
RRN = RR.RobotRaconteurNode.s

node_setup = RR.ClientNodeSetup(argv=__import__("sys").argv)
node_setup.ReleaseNode()

local_transport = node_setup.local_transport
tcp_transport = node_setup.tcp_transport
hardware_transport = node_setup.hardware_transport
intra_transport = node_setup.intra_transport

__all__ = [
    'RR',
    'RRN',
    'node_setup',
    'local_transport',
    'tcp_transport',
    'hardware_transport',
    'intra_transport',
]



import RobotRaconteur as RR
import pytest
import time
import threading
import traceback

_sub_test_service_def = """
service experimental.subobject_sub_test

object testobj
    objref testobj2 subobj2
    function string getf_service_path()
end

object testobj2
    objref testobj3 subobj3_1
    objref testobj3[] subobj3_2
    objref testobj3{string} subobj3_3
    function string getf_service_path()
end

object testobj3
    function double add_two_numbers(double a, double b)
    function string getf_service_path()
end
"""


class _testobj3_impl():

    def __init__(self, index=None):
        self._service_path = None
        self._index = index

    def RRServiceObjectInit(self, ctx, service_path):
        self._service_path = service_path

    def add_two_numbers(self, a, b):
        return a + b

    def getf_service_path(self):
        return self._service_path


class _testobj2_impl():

    def __init__(self):
        self._service_path = None

    def RRServiceObjectInit(self, ctx, service_path):
        self._service_path = service_path

    def getf_service_path(self):
        return self._service_path

    def get_subobj3_1(self):
        return _testobj3_impl(), "experimental.subobject_sub_test.testobj3"

    def get_subobj3_2(self, ind):
        return _testobj3_impl(ind), "experimental.subobject_sub_test.testobj3"

    def get_subobj3_3(self, ind):
        return _testobj3_impl(ind), "experimental.subobject_sub_test.testobj3"


class _testobj_impl():

    def __init__(self):
        self._service_path = None

    def RRServiceObjectInit(self, ctx, service_path):
        self._service_path = service_path

    def getf_service_path(self):
        return self._service_path

    def get_subobj2(self):
        return _testobj2_impl(), "experimental.subobject_sub_test.testobj2"


class _testservice_impl():

    def __init__(self, nodename, nodeid):
        self._obj = _testobj_impl()
        self._node = RR.RobotRaconteurNode()
        self._node.SetNodeID(nodeid)
        self._node.Init()

        self._node.RegisterServiceType(_sub_test_service_def)
        self._node.RegisterService(
            "test_service", "experimental.subobject_sub_test.testobj", self._obj)

        self._node_setup = RR.RobotRaconteurNodeSetup(
            nodename, 0, flags=intra_server_flags, node=self._node)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._node_setup.ReleaseNode()
        self._node.Shutdown()
        self._node_setup = None
        self._node = None


intra_server_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_INTRA_TRANSPORT_START_SERVER \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_ANNOUNCE \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE

intra_client_flags = RR.RobotRaconteurNodeSetupFlags_ENABLE_INTRA_TRANSPORT \
    | RR.RobotRaconteurNodeSetupFlags_ENABLE_NODE_DISCOVERY_LISTENING \
    | RR.RobotRaconteurNodeSetupFlags_DISABLE_STRINGTABLE


def test_subscription_subobject():

    test_servers = {
        "server1": RR.NodeID("0d694574-1ad8-4b9e-9aea-e881524fb451"),
        "server2": RR.NodeID("e23ac123-4357-467e-b44b-4c9eb4ff7916"),
        "server3": RR.NodeID("cb71939a-6c6c-43cc-b6be-070a76acec74")
    }

    client_node = RR.RobotRaconteurNode()
    client_node.Init()

    client_node_setup = RR.RobotRaconteurNodeSetup(
        "", 0, flags=intra_client_flags, node=client_node)

    server1 = _testservice_impl("server1", test_servers["server1"])

    with client_node_setup, server1:

        sub = client_node.SubscribeServiceByType(
            "experimental.subobject_sub_test.testobj")
        c = sub.GetDefaultClientWait(5)

        assert c.getf_service_path() == "test_service"

        sub2 = sub.SubscribeSubObject("*.subobj2")
        c2 = sub2.GetDefaultClient()
        assert c2.getf_service_path() == "test_service.subobj2"

        c3 = sub2.GetDefaultClientWait(1)
        assert c3.getf_service_path() == "test_service.subobj2"

        res, c4 = sub2.TryGetDefaultClient()
        assert res and c4.getf_service_path() == "test_service.subobj2"

        res, c5 = sub2.TryGetDefaultClientWait(1)
        assert res and c5.getf_service_path() == "test_service.subobj2"

        evt = threading.Event()

        def handler(c, exp):
            try:
                if exp is not None:
                    raise exp
                assert c.getf_service_path() == "test_service.subobj2"
                print("Got async callback")
                evt.set()
            except:
                traceback.print_exc()
                raise

        sub2.AsyncGetDefaultClient(handler, 1)
        assert evt.wait(1)

        sub3 = sub.SubscribeSubObject("*.subobj2.subobj3_1")
        c6 = sub3.GetDefaultClient()
        assert c6.getf_service_path() == "test_service.subobj2.subobj3_1"

        sub4 = sub.SubscribeSubObject("*.subobj2.subobj3_2[123]")
        c7 = sub4.GetDefaultClient()
        assert c7.getf_service_path() == "test_service.subobj2.subobj3_2[123]"

        sub5 = sub.SubscribeSubObject("*.subobj2.subobj3_3[someobj]")
        c8 = sub5.GetDefaultClient()
        assert c8.getf_service_path() == "test_service.subobj2.subobj3_3[someobj]"



from pytest_pyodide import run_in_pyodide


@run_in_pyodide(packages=["RobotRaconteur", "numpy"])
async def test_timer(selenium):
    import asyncio
    import RobotRaconteur as RR
    RRN = RR.RobotRaconteurNode.s

    evt = asyncio.Event()

    i = [0]

    def timer_cb(ev):

        if ev.stopped:
            return
        print("Got timer callback!")
        i[0] += 1
        if i[0] > 10:
            timer.TryStop()
            evt.set()
            print("Stopped")

    timer = RRN.CreateTimer(0.1, timer_cb, False)
    timer.Start()


@run_in_pyodide(packages=["RobotRaconteur", "numpy"])
async def test_post(selenium):
    import asyncio
    import RobotRaconteur as RR
    RRN = RR.RobotRaconteurNode.s

    evt = asyncio.Event()

    def post_cb():

        print("Got post callback")
        evt.set()

    RRN.PostToThreadPool(post_cb)

    assert await asyncio.wait_for(evt.wait(), 5)



# Simple example Robot Raconteur webcam client
# This program will capture a frame from both webcams and show it
# on the screen

from RobotRaconteur.Client import *
import time
import numpy
import cv2
import sys

# Main program


def main():

    url = 'rr+tcp://localhost:22355?service=webcam'
    if (len(sys.argv) >= 2):
        url = sys.argv[1]

    # Start up Robot Raconteur and connect, standard by this point
    c1 = RRN.ConnectService(url)

    # Save image to buffer
    size = c1.capture_frame_to_buffer()

    # Read the data from the "buffer "memory. For this example just read the
    # entire buffer
    l = c1.buffer.Length
    data = numpy.zeros(l, dtype="u1")

    c1.buffer.Read(0, data, 0, l)

    frame1 = data.reshape([size.height, size.width, 3], order='C')
    cv2.imshow("buffer", frame1)

    # Read segment from the "multidimbuffer" and display the "red" channel
    bufsize = c1.multidimbuffer.Dimensions
    print(bufsize)

    # create a smaller buffer with 1 channel
    segdata = numpy.zeros([100, 100, 1], dtype="u1")
    c1.multidimbuffer.Read([10, 10, 0], segdata, [0, 0, 0], [100, 100, 1])

    cv2.imshow("multidimbuffer", segdata)

    # CV wait for key press on the image window and then destroy
    cv2.waitKey(0)
    cv2.destroyAllWindows()


if __name__ == '__main__':
    main()



from __future__ import absolute_import

from .RobotRaconteurPython import *
from .RobotRaconteurPythonError import *
from .RobotRaconteurPythonUtil import PipeEndpoint, Pipe, PipeBroadcaster, \
    Wire, WireConnection, WireBroadcaster, WireUnicastReceiver, \
    ServiceSubscriptionFilter, ServiceSubscriptionFilterNode, RobotRaconteurNodeSetup, \
    ClientNodeSetup, ServerNodeSetup, SecureServerNodeSetup, UserLogRecordHandler, TapFileReader, BroadcastDownsamplerStep, \
    ServiceInfo2, NodeInfo2, ServiceInfo2Subscription, ServiceSubscription, WireSubscription, PipeSubscription, \
    ServiceSubscriptionClientID, SubObjectSubscription, ServiceSubscriptionManager, ServiceSubscriptionManagerDetails
from .RobotRaconteurPythonDataTypes import *



# reynard_quickstart_client_sub.py

from RobotRaconteur.Client import *

# RRN is imported from RobotRaconteur.Client
# Create a subscription and connect to the default client
sub = RRN.SubscribeService('rr+tcp://localhost:53222/?service=reynard')
obj = sub.GetDefaultClientWait(5)

# Call the say function
obj.say("Hello from Reynard!")

# Call the teleport function
obj.teleport(100, 200)
